import cffi as _cffi
import pkg_resources as _pkg_resources
import collections as _collections
import weakref as _weakref
import sys

class PlatformNotSupportedError(Exception):
	pass

class ProcedureNotFoundError(Exception):
	pass

class ExtensionNotSupportedError(Exception):
	pass

ffi = _cffi.FFI()

_weakkey_dict = _weakref.WeakKeyDictionary()

def _castToPtr2(x, _type):
	if isinstance(x, ffi.CData):
		if _type.item==ffi.typeof(x) or (_type.item.cname=='void' and ffi.typeof(x).kind in ['struct', 'union']):
			return ffi.addressof(x), x
		return x, x
	if isinstance(x, _collections.Iterable):
		if _type.item.kind=='pointer':
			ptrs = [_castToPtr(i, _type.item) for i in x]
			ret = ffi.new(_type.item.cname+'[]', [i for i, _ in ptrs])
			_weakkey_dict[ret] = tuple(i for _, i in ptrs if i!=ffi.NULL)
		else:
			ret = ffi.new(_type.item.cname+'[]', x)
		return ret, ret
	return ffi.cast(_type, x), x

def _castToPtr3(x, _type):
	if isinstance(x, str):
		x = x.encode('ascii')
	return _castToPtr2(x, _type)

if sys.version_info<(3, 0):
	def _cdef(header):
		ffi.cdef(_pkg_resources.resource_string(__name__, header))
	_castToPtr = _castToPtr2
else:
	def _cdef(header):
		ffi.cdef(_pkg_resources.resource_string(__name__, header).decode())
	_castToPtr = _castToPtr3

if sys.platform=='win32':
	_cdef('vulkan_win32_cffi.h')
	_lib = ffi.dlopen('vulkan-1.dll')
elif sys.platform.startswith('linux'):
	_cdef('vulkan_linux_cffi.h')
	_lib = ffi.dlopen('libvulkan.so')
else:
	raise PlatformNotSupportedError()

VK_ATTACHMENT_LOAD_OP_LOAD = 0
VK_ATTACHMENT_LOAD_OP_CLEAR = 1
VK_ATTACHMENT_LOAD_OP_DONT_CARE = 2
VK_ATTACHMENT_STORE_OP_STORE = 0
VK_ATTACHMENT_STORE_OP_DONT_CARE = 1
VK_BLEND_FACTOR_ZERO = 0
VK_BLEND_FACTOR_ONE = 1
VK_BLEND_FACTOR_SRC_COLOR = 2
VK_BLEND_FACTOR_ONE_MINUS_SRC_COLOR = 3
VK_BLEND_FACTOR_DST_COLOR = 4
VK_BLEND_FACTOR_ONE_MINUS_DST_COLOR = 5
VK_BLEND_FACTOR_SRC_ALPHA = 6
VK_BLEND_FACTOR_ONE_MINUS_SRC_ALPHA = 7
VK_BLEND_FACTOR_DST_ALPHA = 8
VK_BLEND_FACTOR_ONE_MINUS_DST_ALPHA = 9
VK_BLEND_FACTOR_CONSTANT_COLOR = 10
VK_BLEND_FACTOR_ONE_MINUS_CONSTANT_COLOR = 11
VK_BLEND_FACTOR_CONSTANT_ALPHA = 12
VK_BLEND_FACTOR_ONE_MINUS_CONSTANT_ALPHA = 13
VK_BLEND_FACTOR_SRC_ALPHA_SATURATE = 14
VK_BLEND_FACTOR_SRC1_COLOR = 15
VK_BLEND_FACTOR_ONE_MINUS_SRC1_COLOR = 16
VK_BLEND_FACTOR_SRC1_ALPHA = 17
VK_BLEND_FACTOR_ONE_MINUS_SRC1_ALPHA = 18
VK_BLEND_OP_ADD = 0
VK_BLEND_OP_SUBTRACT = 1
VK_BLEND_OP_REVERSE_SUBTRACT = 2
VK_BLEND_OP_MIN = 3
VK_BLEND_OP_MAX = 4
VK_BORDER_COLOR_FLOAT_TRANSPARENT_BLACK = 0
VK_BORDER_COLOR_INT_TRANSPARENT_BLACK = 1
VK_BORDER_COLOR_FLOAT_OPAQUE_BLACK = 2
VK_BORDER_COLOR_INT_OPAQUE_BLACK = 3
VK_BORDER_COLOR_FLOAT_OPAQUE_WHITE = 4
VK_BORDER_COLOR_INT_OPAQUE_WHITE = 5
VK_PIPELINE_CACHE_HEADER_VERSION_ONE = 1
VK_BUFFER_CREATE_SPARSE_BINDING_BIT = 1
VK_BUFFER_CREATE_SPARSE_RESIDENCY_BIT = 2
VK_BUFFER_CREATE_SPARSE_ALIASED_BIT = 4
VK_BUFFER_USAGE_TRANSFER_SRC_BIT = 1
VK_BUFFER_USAGE_TRANSFER_DST_BIT = 2
VK_BUFFER_USAGE_UNIFORM_TEXEL_BUFFER_BIT = 4
VK_BUFFER_USAGE_STORAGE_TEXEL_BUFFER_BIT = 8
VK_BUFFER_USAGE_UNIFORM_BUFFER_BIT = 16
VK_BUFFER_USAGE_STORAGE_BUFFER_BIT = 32
VK_BUFFER_USAGE_INDEX_BUFFER_BIT = 64
VK_BUFFER_USAGE_VERTEX_BUFFER_BIT = 128
VK_BUFFER_USAGE_INDIRECT_BUFFER_BIT = 256
VK_COLOR_COMPONENT_R_BIT = 1
VK_COLOR_COMPONENT_G_BIT = 2
VK_COLOR_COMPONENT_B_BIT = 4
VK_COLOR_COMPONENT_A_BIT = 8
VK_COMPONENT_SWIZZLE_IDENTITY = 0
VK_COMPONENT_SWIZZLE_ZERO = 1
VK_COMPONENT_SWIZZLE_ONE = 2
VK_COMPONENT_SWIZZLE_R = 3
VK_COMPONENT_SWIZZLE_G = 4
VK_COMPONENT_SWIZZLE_B = 5
VK_COMPONENT_SWIZZLE_A = 6
VK_COMMAND_POOL_CREATE_TRANSIENT_BIT = 1
VK_COMMAND_POOL_CREATE_RESET_COMMAND_BUFFER_BIT = 2
VK_COMMAND_POOL_RESET_RELEASE_RESOURCES_BIT = 1
VK_COMMAND_BUFFER_RESET_RELEASE_RESOURCES_BIT = 1
VK_COMMAND_BUFFER_LEVEL_PRIMARY = 0
VK_COMMAND_BUFFER_LEVEL_SECONDARY = 1
VK_COMMAND_BUFFER_USAGE_ONE_TIME_SUBMIT_BIT = 1
VK_COMMAND_BUFFER_USAGE_RENDER_PASS_CONTINUE_BIT = 2
VK_COMMAND_BUFFER_USAGE_SIMULTANEOUS_USE_BIT = 4
VK_COMPARE_OP_NEVER = 0
VK_COMPARE_OP_LESS = 1
VK_COMPARE_OP_EQUAL = 2
VK_COMPARE_OP_LESS_OR_EQUAL = 3
VK_COMPARE_OP_GREATER = 4
VK_COMPARE_OP_NOT_EQUAL = 5
VK_COMPARE_OP_GREATER_OR_EQUAL = 6
VK_COMPARE_OP_ALWAYS = 7
VK_CULL_MODE_NONE = 0
VK_CULL_MODE_FRONT_BIT = 1
VK_CULL_MODE_BACK_BIT = 2
VK_CULL_MODE_FRONT_AND_BACK = 0x00000003
VK_DESCRIPTOR_TYPE_SAMPLER = 0
VK_DESCRIPTOR_TYPE_COMBINED_IMAGE_SAMPLER = 1
VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE = 2
VK_DESCRIPTOR_TYPE_STORAGE_IMAGE = 3
VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER = 4
VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER = 5
VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER = 6
VK_DESCRIPTOR_TYPE_STORAGE_BUFFER = 7
VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC = 8
VK_DESCRIPTOR_TYPE_STORAGE_BUFFER_DYNAMIC = 9
VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT = 10
VK_DYNAMIC_STATE_VIEWPORT = 0
VK_DYNAMIC_STATE_SCISSOR = 1
VK_DYNAMIC_STATE_LINE_WIDTH = 2
VK_DYNAMIC_STATE_DEPTH_BIAS = 3
VK_DYNAMIC_STATE_BLEND_CONSTANTS = 4
VK_DYNAMIC_STATE_DEPTH_BOUNDS = 5
VK_DYNAMIC_STATE_STENCIL_COMPARE_MASK = 6
VK_DYNAMIC_STATE_STENCIL_WRITE_MASK = 7
VK_DYNAMIC_STATE_STENCIL_REFERENCE = 8
VK_FENCE_CREATE_SIGNALED_BIT = 1
VK_POLYGON_MODE_FILL = 0
VK_POLYGON_MODE_LINE = 1
VK_POLYGON_MODE_POINT = 2
VK_FORMAT_UNDEFINED = 0
VK_FORMAT_R4G4_UNORM_PACK8 = 1
VK_FORMAT_R4G4B4A4_UNORM_PACK16 = 2
VK_FORMAT_B4G4R4A4_UNORM_PACK16 = 3
VK_FORMAT_R5G6B5_UNORM_PACK16 = 4
VK_FORMAT_B5G6R5_UNORM_PACK16 = 5
VK_FORMAT_R5G5B5A1_UNORM_PACK16 = 6
VK_FORMAT_B5G5R5A1_UNORM_PACK16 = 7
VK_FORMAT_A1R5G5B5_UNORM_PACK16 = 8
VK_FORMAT_R8_UNORM = 9
VK_FORMAT_R8_SNORM = 10
VK_FORMAT_R8_USCALED = 11
VK_FORMAT_R8_SSCALED = 12
VK_FORMAT_R8_UINT = 13
VK_FORMAT_R8_SINT = 14
VK_FORMAT_R8_SRGB = 15
VK_FORMAT_R8G8_UNORM = 16
VK_FORMAT_R8G8_SNORM = 17
VK_FORMAT_R8G8_USCALED = 18
VK_FORMAT_R8G8_SSCALED = 19
VK_FORMAT_R8G8_UINT = 20
VK_FORMAT_R8G8_SINT = 21
VK_FORMAT_R8G8_SRGB = 22
VK_FORMAT_R8G8B8_UNORM = 23
VK_FORMAT_R8G8B8_SNORM = 24
VK_FORMAT_R8G8B8_USCALED = 25
VK_FORMAT_R8G8B8_SSCALED = 26
VK_FORMAT_R8G8B8_UINT = 27
VK_FORMAT_R8G8B8_SINT = 28
VK_FORMAT_R8G8B8_SRGB = 29
VK_FORMAT_B8G8R8_UNORM = 30
VK_FORMAT_B8G8R8_SNORM = 31
VK_FORMAT_B8G8R8_USCALED = 32
VK_FORMAT_B8G8R8_SSCALED = 33
VK_FORMAT_B8G8R8_UINT = 34
VK_FORMAT_B8G8R8_SINT = 35
VK_FORMAT_B8G8R8_SRGB = 36
VK_FORMAT_R8G8B8A8_UNORM = 37
VK_FORMAT_R8G8B8A8_SNORM = 38
VK_FORMAT_R8G8B8A8_USCALED = 39
VK_FORMAT_R8G8B8A8_SSCALED = 40
VK_FORMAT_R8G8B8A8_UINT = 41
VK_FORMAT_R8G8B8A8_SINT = 42
VK_FORMAT_R8G8B8A8_SRGB = 43
VK_FORMAT_B8G8R8A8_UNORM = 44
VK_FORMAT_B8G8R8A8_SNORM = 45
VK_FORMAT_B8G8R8A8_USCALED = 46
VK_FORMAT_B8G8R8A8_SSCALED = 47
VK_FORMAT_B8G8R8A8_UINT = 48
VK_FORMAT_B8G8R8A8_SINT = 49
VK_FORMAT_B8G8R8A8_SRGB = 50
VK_FORMAT_A8B8G8R8_UNORM_PACK32 = 51
VK_FORMAT_A8B8G8R8_SNORM_PACK32 = 52
VK_FORMAT_A8B8G8R8_USCALED_PACK32 = 53
VK_FORMAT_A8B8G8R8_SSCALED_PACK32 = 54
VK_FORMAT_A8B8G8R8_UINT_PACK32 = 55
VK_FORMAT_A8B8G8R8_SINT_PACK32 = 56
VK_FORMAT_A8B8G8R8_SRGB_PACK32 = 57
VK_FORMAT_A2R10G10B10_UNORM_PACK32 = 58
VK_FORMAT_A2R10G10B10_SNORM_PACK32 = 59
VK_FORMAT_A2R10G10B10_USCALED_PACK32 = 60
VK_FORMAT_A2R10G10B10_SSCALED_PACK32 = 61
VK_FORMAT_A2R10G10B10_UINT_PACK32 = 62
VK_FORMAT_A2R10G10B10_SINT_PACK32 = 63
VK_FORMAT_A2B10G10R10_UNORM_PACK32 = 64
VK_FORMAT_A2B10G10R10_SNORM_PACK32 = 65
VK_FORMAT_A2B10G10R10_USCALED_PACK32 = 66
VK_FORMAT_A2B10G10R10_SSCALED_PACK32 = 67
VK_FORMAT_A2B10G10R10_UINT_PACK32 = 68
VK_FORMAT_A2B10G10R10_SINT_PACK32 = 69
VK_FORMAT_R16_UNORM = 70
VK_FORMAT_R16_SNORM = 71
VK_FORMAT_R16_USCALED = 72
VK_FORMAT_R16_SSCALED = 73
VK_FORMAT_R16_UINT = 74
VK_FORMAT_R16_SINT = 75
VK_FORMAT_R16_SFLOAT = 76
VK_FORMAT_R16G16_UNORM = 77
VK_FORMAT_R16G16_SNORM = 78
VK_FORMAT_R16G16_USCALED = 79
VK_FORMAT_R16G16_SSCALED = 80
VK_FORMAT_R16G16_UINT = 81
VK_FORMAT_R16G16_SINT = 82
VK_FORMAT_R16G16_SFLOAT = 83
VK_FORMAT_R16G16B16_UNORM = 84
VK_FORMAT_R16G16B16_SNORM = 85
VK_FORMAT_R16G16B16_USCALED = 86
VK_FORMAT_R16G16B16_SSCALED = 87
VK_FORMAT_R16G16B16_UINT = 88
VK_FORMAT_R16G16B16_SINT = 89
VK_FORMAT_R16G16B16_SFLOAT = 90
VK_FORMAT_R16G16B16A16_UNORM = 91
VK_FORMAT_R16G16B16A16_SNORM = 92
VK_FORMAT_R16G16B16A16_USCALED = 93
VK_FORMAT_R16G16B16A16_SSCALED = 94
VK_FORMAT_R16G16B16A16_UINT = 95
VK_FORMAT_R16G16B16A16_SINT = 96
VK_FORMAT_R16G16B16A16_SFLOAT = 97
VK_FORMAT_R32_UINT = 98
VK_FORMAT_R32_SINT = 99
VK_FORMAT_R32_SFLOAT = 100
VK_FORMAT_R32G32_UINT = 101
VK_FORMAT_R32G32_SINT = 102
VK_FORMAT_R32G32_SFLOAT = 103
VK_FORMAT_R32G32B32_UINT = 104
VK_FORMAT_R32G32B32_SINT = 105
VK_FORMAT_R32G32B32_SFLOAT = 106
VK_FORMAT_R32G32B32A32_UINT = 107
VK_FORMAT_R32G32B32A32_SINT = 108
VK_FORMAT_R32G32B32A32_SFLOAT = 109
VK_FORMAT_R64_UINT = 110
VK_FORMAT_R64_SINT = 111
VK_FORMAT_R64_SFLOAT = 112
VK_FORMAT_R64G64_UINT = 113
VK_FORMAT_R64G64_SINT = 114
VK_FORMAT_R64G64_SFLOAT = 115
VK_FORMAT_R64G64B64_UINT = 116
VK_FORMAT_R64G64B64_SINT = 117
VK_FORMAT_R64G64B64_SFLOAT = 118
VK_FORMAT_R64G64B64A64_UINT = 119
VK_FORMAT_R64G64B64A64_SINT = 120
VK_FORMAT_R64G64B64A64_SFLOAT = 121
VK_FORMAT_B10G11R11_UFLOAT_PACK32 = 122
VK_FORMAT_E5B9G9R9_UFLOAT_PACK32 = 123
VK_FORMAT_D16_UNORM = 124
VK_FORMAT_X8_D24_UNORM_PACK32 = 125
VK_FORMAT_D32_SFLOAT = 126
VK_FORMAT_S8_UINT = 127
VK_FORMAT_D16_UNORM_S8_UINT = 128
VK_FORMAT_D24_UNORM_S8_UINT = 129
VK_FORMAT_D32_SFLOAT_S8_UINT = 130
VK_FORMAT_BC1_RGB_UNORM_BLOCK = 131
VK_FORMAT_BC1_RGB_SRGB_BLOCK = 132
VK_FORMAT_BC1_RGBA_UNORM_BLOCK = 133
VK_FORMAT_BC1_RGBA_SRGB_BLOCK = 134
VK_FORMAT_BC2_UNORM_BLOCK = 135
VK_FORMAT_BC2_SRGB_BLOCK = 136
VK_FORMAT_BC3_UNORM_BLOCK = 137
VK_FORMAT_BC3_SRGB_BLOCK = 138
VK_FORMAT_BC4_UNORM_BLOCK = 139
VK_FORMAT_BC4_SNORM_BLOCK = 140
VK_FORMAT_BC5_UNORM_BLOCK = 141
VK_FORMAT_BC5_SNORM_BLOCK = 142
VK_FORMAT_BC6H_UFLOAT_BLOCK = 143
VK_FORMAT_BC6H_SFLOAT_BLOCK = 144
VK_FORMAT_BC7_UNORM_BLOCK = 145
VK_FORMAT_BC7_SRGB_BLOCK = 146
VK_FORMAT_ETC2_R8G8B8_UNORM_BLOCK = 147
VK_FORMAT_ETC2_R8G8B8_SRGB_BLOCK = 148
VK_FORMAT_ETC2_R8G8B8A1_UNORM_BLOCK = 149
VK_FORMAT_ETC2_R8G8B8A1_SRGB_BLOCK = 150
VK_FORMAT_ETC2_R8G8B8A8_UNORM_BLOCK = 151
VK_FORMAT_ETC2_R8G8B8A8_SRGB_BLOCK = 152
VK_FORMAT_EAC_R11_UNORM_BLOCK = 153
VK_FORMAT_EAC_R11_SNORM_BLOCK = 154
VK_FORMAT_EAC_R11G11_UNORM_BLOCK = 155
VK_FORMAT_EAC_R11G11_SNORM_BLOCK = 156
VK_FORMAT_ASTC_4x4_UNORM_BLOCK = 157
VK_FORMAT_ASTC_4x4_SRGB_BLOCK = 158
VK_FORMAT_ASTC_5x4_UNORM_BLOCK = 159
VK_FORMAT_ASTC_5x4_SRGB_BLOCK = 160
VK_FORMAT_ASTC_5x5_UNORM_BLOCK = 161
VK_FORMAT_ASTC_5x5_SRGB_BLOCK = 162
VK_FORMAT_ASTC_6x5_UNORM_BLOCK = 163
VK_FORMAT_ASTC_6x5_SRGB_BLOCK = 164
VK_FORMAT_ASTC_6x6_UNORM_BLOCK = 165
VK_FORMAT_ASTC_6x6_SRGB_BLOCK = 166
VK_FORMAT_ASTC_8x5_UNORM_BLOCK = 167
VK_FORMAT_ASTC_8x5_SRGB_BLOCK = 168
VK_FORMAT_ASTC_8x6_UNORM_BLOCK = 169
VK_FORMAT_ASTC_8x6_SRGB_BLOCK = 170
VK_FORMAT_ASTC_8x8_UNORM_BLOCK = 171
VK_FORMAT_ASTC_8x8_SRGB_BLOCK = 172
VK_FORMAT_ASTC_10x5_UNORM_BLOCK = 173
VK_FORMAT_ASTC_10x5_SRGB_BLOCK = 174
VK_FORMAT_ASTC_10x6_UNORM_BLOCK = 175
VK_FORMAT_ASTC_10x6_SRGB_BLOCK = 176
VK_FORMAT_ASTC_10x8_UNORM_BLOCK = 177
VK_FORMAT_ASTC_10x8_SRGB_BLOCK = 178
VK_FORMAT_ASTC_10x10_UNORM_BLOCK = 179
VK_FORMAT_ASTC_10x10_SRGB_BLOCK = 180
VK_FORMAT_ASTC_12x10_UNORM_BLOCK = 181
VK_FORMAT_ASTC_12x10_SRGB_BLOCK = 182
VK_FORMAT_ASTC_12x12_UNORM_BLOCK = 183
VK_FORMAT_ASTC_12x12_SRGB_BLOCK = 184
VK_FORMAT_PVRTC1_2BPP_UNORM_BLOCK_IMG = 1000054000
VK_FORMAT_PVRTC1_4BPP_UNORM_BLOCK_IMG = 1000054001
VK_FORMAT_PVRTC2_2BPP_UNORM_BLOCK_IMG = 1000054002
VK_FORMAT_PVRTC2_4BPP_UNORM_BLOCK_IMG = 1000054003
VK_FORMAT_PVRTC1_2BPP_SRGB_BLOCK_IMG = 1000054004
VK_FORMAT_PVRTC1_4BPP_SRGB_BLOCK_IMG = 1000054005
VK_FORMAT_PVRTC2_2BPP_SRGB_BLOCK_IMG = 1000054006
VK_FORMAT_PVRTC2_4BPP_SRGB_BLOCK_IMG = 1000054007
VK_FORMAT_FEATURE_SAMPLED_IMAGE_BIT = 1
VK_FORMAT_FEATURE_STORAGE_IMAGE_BIT = 2
VK_FORMAT_FEATURE_STORAGE_IMAGE_ATOMIC_BIT = 4
VK_FORMAT_FEATURE_UNIFORM_TEXEL_BUFFER_BIT = 8
VK_FORMAT_FEATURE_STORAGE_TEXEL_BUFFER_BIT = 16
VK_FORMAT_FEATURE_STORAGE_TEXEL_BUFFER_ATOMIC_BIT = 32
VK_FORMAT_FEATURE_VERTEX_BUFFER_BIT = 64
VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BIT = 128
VK_FORMAT_FEATURE_COLOR_ATTACHMENT_BLEND_BIT = 256
VK_FORMAT_FEATURE_DEPTH_STENCIL_ATTACHMENT_BIT = 512
VK_FORMAT_FEATURE_BLIT_SRC_BIT = 1024
VK_FORMAT_FEATURE_BLIT_DST_BIT = 2048
VK_FORMAT_FEATURE_SAMPLED_IMAGE_FILTER_LINEAR_BIT = 4096
VK_FORMAT_FEATURE_SAMPLED_IMAGE_FILTER_CUBIC_BIT_IMG = 8192
VK_FRONT_FACE_COUNTER_CLOCKWISE = 0
VK_FRONT_FACE_CLOCKWISE = 1
VK_IMAGE_ASPECT_COLOR_BIT = 1
VK_IMAGE_ASPECT_DEPTH_BIT = 2
VK_IMAGE_ASPECT_STENCIL_BIT = 4
VK_IMAGE_ASPECT_METADATA_BIT = 8
VK_IMAGE_CREATE_SPARSE_BINDING_BIT = 1
VK_IMAGE_CREATE_SPARSE_RESIDENCY_BIT = 2
VK_IMAGE_CREATE_SPARSE_ALIASED_BIT = 4
VK_IMAGE_CREATE_MUTABLE_FORMAT_BIT = 8
VK_IMAGE_CREATE_CUBE_COMPATIBLE_BIT = 16
VK_IMAGE_LAYOUT_UNDEFINED = 0
VK_IMAGE_LAYOUT_GENERAL = 1
VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL = 2
VK_IMAGE_LAYOUT_DEPTH_STENCIL_ATTACHMENT_OPTIMAL = 3
VK_IMAGE_LAYOUT_DEPTH_STENCIL_READ_ONLY_OPTIMAL = 4
VK_IMAGE_LAYOUT_SHADER_READ_ONLY_OPTIMAL = 5
VK_IMAGE_LAYOUT_TRANSFER_SRC_OPTIMAL = 6
VK_IMAGE_LAYOUT_TRANSFER_DST_OPTIMAL = 7
VK_IMAGE_LAYOUT_PREINITIALIZED = 8
VK_IMAGE_LAYOUT_PRESENT_SRC_KHR = 1000001002
VK_IMAGE_TILING_OPTIMAL = 0
VK_IMAGE_TILING_LINEAR = 1
VK_IMAGE_TYPE_1D = 0
VK_IMAGE_TYPE_2D = 1
VK_IMAGE_TYPE_3D = 2
VK_IMAGE_USAGE_TRANSFER_SRC_BIT = 1
VK_IMAGE_USAGE_TRANSFER_DST_BIT = 2
VK_IMAGE_USAGE_SAMPLED_BIT = 4
VK_IMAGE_USAGE_STORAGE_BIT = 8
VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT = 16
VK_IMAGE_USAGE_DEPTH_STENCIL_ATTACHMENT_BIT = 32
VK_IMAGE_USAGE_TRANSIENT_ATTACHMENT_BIT = 64
VK_IMAGE_USAGE_INPUT_ATTACHMENT_BIT = 128
VK_IMAGE_VIEW_TYPE_1D = 0
VK_IMAGE_VIEW_TYPE_2D = 1
VK_IMAGE_VIEW_TYPE_3D = 2
VK_IMAGE_VIEW_TYPE_CUBE = 3
VK_IMAGE_VIEW_TYPE_1D_ARRAY = 4
VK_IMAGE_VIEW_TYPE_2D_ARRAY = 5
VK_IMAGE_VIEW_TYPE_CUBE_ARRAY = 6
VK_SHARING_MODE_EXCLUSIVE = 0
VK_SHARING_MODE_CONCURRENT = 1
VK_INDEX_TYPE_UINT16 = 0
VK_INDEX_TYPE_UINT32 = 1
VK_LOGIC_OP_CLEAR = 0
VK_LOGIC_OP_AND = 1
VK_LOGIC_OP_AND_REVERSE = 2
VK_LOGIC_OP_COPY = 3
VK_LOGIC_OP_AND_INVERTED = 4
VK_LOGIC_OP_NO_OP = 5
VK_LOGIC_OP_XOR = 6
VK_LOGIC_OP_OR = 7
VK_LOGIC_OP_NOR = 8
VK_LOGIC_OP_EQUIVALENT = 9
VK_LOGIC_OP_INVERT = 10
VK_LOGIC_OP_OR_REVERSE = 11
VK_LOGIC_OP_COPY_INVERTED = 12
VK_LOGIC_OP_OR_INVERTED = 13
VK_LOGIC_OP_NAND = 14
VK_LOGIC_OP_SET = 15
VK_MEMORY_HEAP_DEVICE_LOCAL_BIT = 1
VK_ACCESS_INDIRECT_COMMAND_READ_BIT = 1
VK_ACCESS_INDEX_READ_BIT = 2
VK_ACCESS_VERTEX_ATTRIBUTE_READ_BIT = 4
VK_ACCESS_UNIFORM_READ_BIT = 8
VK_ACCESS_INPUT_ATTACHMENT_READ_BIT = 16
VK_ACCESS_SHADER_READ_BIT = 32
VK_ACCESS_SHADER_WRITE_BIT = 64
VK_ACCESS_COLOR_ATTACHMENT_READ_BIT = 128
VK_ACCESS_COLOR_ATTACHMENT_WRITE_BIT = 256
VK_ACCESS_DEPTH_STENCIL_ATTACHMENT_READ_BIT = 512
VK_ACCESS_DEPTH_STENCIL_ATTACHMENT_WRITE_BIT = 1024
VK_ACCESS_TRANSFER_READ_BIT = 2048
VK_ACCESS_TRANSFER_WRITE_BIT = 4096
VK_ACCESS_HOST_READ_BIT = 8192
VK_ACCESS_HOST_WRITE_BIT = 16384
VK_ACCESS_MEMORY_READ_BIT = 32768
VK_ACCESS_MEMORY_WRITE_BIT = 65536
VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT = 1
VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT = 2
VK_MEMORY_PROPERTY_HOST_COHERENT_BIT = 4
VK_MEMORY_PROPERTY_HOST_CACHED_BIT = 8
VK_MEMORY_PROPERTY_LAZILY_ALLOCATED_BIT = 16
VK_PHYSICAL_DEVICE_TYPE_OTHER = 0
VK_PHYSICAL_DEVICE_TYPE_INTEGRATED_GPU = 1
VK_PHYSICAL_DEVICE_TYPE_DISCRETE_GPU = 2
VK_PHYSICAL_DEVICE_TYPE_VIRTUAL_GPU = 3
VK_PHYSICAL_DEVICE_TYPE_CPU = 4
VK_PIPELINE_BIND_POINT_GRAPHICS = 0
VK_PIPELINE_BIND_POINT_COMPUTE = 1
VK_PIPELINE_CREATE_DISABLE_OPTIMIZATION_BIT = 1
VK_PIPELINE_CREATE_ALLOW_DERIVATIVES_BIT = 2
VK_PIPELINE_CREATE_DERIVATIVE_BIT = 4
VK_PRIMITIVE_TOPOLOGY_POINT_LIST = 0
VK_PRIMITIVE_TOPOLOGY_LINE_LIST = 1
VK_PRIMITIVE_TOPOLOGY_LINE_STRIP = 2
VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST = 3
VK_PRIMITIVE_TOPOLOGY_TRIANGLE_STRIP = 4
VK_PRIMITIVE_TOPOLOGY_TRIANGLE_FAN = 5
VK_PRIMITIVE_TOPOLOGY_LINE_LIST_WITH_ADJACENCY = 6
VK_PRIMITIVE_TOPOLOGY_LINE_STRIP_WITH_ADJACENCY = 7
VK_PRIMITIVE_TOPOLOGY_TRIANGLE_LIST_WITH_ADJACENCY = 8
VK_PRIMITIVE_TOPOLOGY_TRIANGLE_STRIP_WITH_ADJACENCY = 9
VK_PRIMITIVE_TOPOLOGY_PATCH_LIST = 10
VK_QUERY_CONTROL_PRECISE_BIT = 1
VK_QUERY_PIPELINE_STATISTIC_INPUT_ASSEMBLY_VERTICES_BIT = 1
VK_QUERY_PIPELINE_STATISTIC_INPUT_ASSEMBLY_PRIMITIVES_BIT = 2
VK_QUERY_PIPELINE_STATISTIC_VERTEX_SHADER_INVOCATIONS_BIT = 4
VK_QUERY_PIPELINE_STATISTIC_GEOMETRY_SHADER_INVOCATIONS_BIT = 8
VK_QUERY_PIPELINE_STATISTIC_GEOMETRY_SHADER_PRIMITIVES_BIT = 16
VK_QUERY_PIPELINE_STATISTIC_CLIPPING_INVOCATIONS_BIT = 32
VK_QUERY_PIPELINE_STATISTIC_CLIPPING_PRIMITIVES_BIT = 64
VK_QUERY_PIPELINE_STATISTIC_FRAGMENT_SHADER_INVOCATIONS_BIT = 128
VK_QUERY_PIPELINE_STATISTIC_TESSELLATION_CONTROL_SHADER_PATCHES_BIT = 256
VK_QUERY_PIPELINE_STATISTIC_TESSELLATION_EVALUATION_SHADER_INVOCATIONS_BIT = 512
VK_QUERY_PIPELINE_STATISTIC_COMPUTE_SHADER_INVOCATIONS_BIT = 1024
VK_QUERY_RESULT_64_BIT = 1
VK_QUERY_RESULT_WAIT_BIT = 2
VK_QUERY_RESULT_WITH_AVAILABILITY_BIT = 4
VK_QUERY_RESULT_PARTIAL_BIT = 8
VK_QUERY_TYPE_OCCLUSION = 0
VK_QUERY_TYPE_PIPELINE_STATISTICS = 1
VK_QUERY_TYPE_TIMESTAMP = 2
VK_QUEUE_GRAPHICS_BIT = 1
VK_QUEUE_COMPUTE_BIT = 2
VK_QUEUE_TRANSFER_BIT = 4
VK_QUEUE_SPARSE_BINDING_BIT = 8
VK_SUBPASS_CONTENTS_INLINE = 0
VK_SUBPASS_CONTENTS_SECONDARY_COMMAND_BUFFERS = 1
VK_SUCCESS = 0
VK_NOT_READY = 1
VK_TIMEOUT = 2
VK_EVENT_SET = 3
VK_EVENT_RESET = 4
VK_INCOMPLETE = 5
VK_ERROR_OUT_OF_HOST_MEMORY = -1
VK_ERROR_OUT_OF_DEVICE_MEMORY = -2
VK_ERROR_INITIALIZATION_FAILED = -3
VK_ERROR_DEVICE_LOST = -4
VK_ERROR_MEMORY_MAP_FAILED = -5
VK_ERROR_LAYER_NOT_PRESENT = -6
VK_ERROR_EXTENSION_NOT_PRESENT = -7
VK_ERROR_FEATURE_NOT_PRESENT = -8
VK_ERROR_INCOMPATIBLE_DRIVER = -9
VK_ERROR_TOO_MANY_OBJECTS = -10
VK_ERROR_FORMAT_NOT_SUPPORTED = -11
VK_ERROR_FRAGMENTED_POOL = -12
VK_ERROR_SURFACE_LOST_KHR = -1000000000
VK_ERROR_NATIVE_WINDOW_IN_USE_KHR = -1000000001
VK_SUBOPTIMAL_KHR = 1000001003
VK_ERROR_OUT_OF_DATE_KHR = -1000001004
VK_ERROR_INCOMPATIBLE_DISPLAY_KHR = -1000003001
VK_ERROR_VALIDATION_FAILED_EXT = -1000011001
VK_ERROR_INVALID_SHADER_NV = -1000012000
VK_SHADER_STAGE_VERTEX_BIT = 1
VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT = 2
VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT = 4
VK_SHADER_STAGE_GEOMETRY_BIT = 8
VK_SHADER_STAGE_FRAGMENT_BIT = 16
VK_SHADER_STAGE_COMPUTE_BIT = 32
VK_SHADER_STAGE_ALL_GRAPHICS = 0x0000001F
VK_SHADER_STAGE_ALL = 0x7FFFFFFF
VK_SPARSE_MEMORY_BIND_METADATA_BIT = 1
VK_STENCIL_FACE_FRONT_BIT = 1
VK_STENCIL_FACE_BACK_BIT = 2
VK_STENCIL_FRONT_AND_BACK = 0x00000003
VK_STENCIL_OP_KEEP = 0
VK_STENCIL_OP_ZERO = 1
VK_STENCIL_OP_REPLACE = 2
VK_STENCIL_OP_INCREMENT_AND_CLAMP = 3
VK_STENCIL_OP_DECREMENT_AND_CLAMP = 4
VK_STENCIL_OP_INVERT = 5
VK_STENCIL_OP_INCREMENT_AND_WRAP = 6
VK_STENCIL_OP_DECREMENT_AND_WRAP = 7
VK_STRUCTURE_TYPE_APPLICATION_INFO = 0
VK_STRUCTURE_TYPE_INSTANCE_CREATE_INFO = 1
VK_STRUCTURE_TYPE_DEVICE_QUEUE_CREATE_INFO = 2
VK_STRUCTURE_TYPE_DEVICE_CREATE_INFO = 3
VK_STRUCTURE_TYPE_SUBMIT_INFO = 4
VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_INFO = 5
VK_STRUCTURE_TYPE_MAPPED_MEMORY_RANGE = 6
VK_STRUCTURE_TYPE_BIND_SPARSE_INFO = 7
VK_STRUCTURE_TYPE_FENCE_CREATE_INFO = 8
VK_STRUCTURE_TYPE_SEMAPHORE_CREATE_INFO = 9
VK_STRUCTURE_TYPE_EVENT_CREATE_INFO = 10
VK_STRUCTURE_TYPE_QUERY_POOL_CREATE_INFO = 11
VK_STRUCTURE_TYPE_BUFFER_CREATE_INFO = 12
VK_STRUCTURE_TYPE_BUFFER_VIEW_CREATE_INFO = 13
VK_STRUCTURE_TYPE_IMAGE_CREATE_INFO = 14
VK_STRUCTURE_TYPE_IMAGE_VIEW_CREATE_INFO = 15
VK_STRUCTURE_TYPE_SHADER_MODULE_CREATE_INFO = 16
VK_STRUCTURE_TYPE_PIPELINE_CACHE_CREATE_INFO = 17
VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO = 18
VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_STATE_CREATE_INFO = 19
VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO = 20
VK_STRUCTURE_TYPE_PIPELINE_TESSELLATION_STATE_CREATE_INFO = 21
VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_STATE_CREATE_INFO = 22
VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO = 23
VK_STRUCTURE_TYPE_PIPELINE_MULTISAMPLE_STATE_CREATE_INFO = 24
VK_STRUCTURE_TYPE_PIPELINE_DEPTH_STENCIL_STATE_CREATE_INFO = 25
VK_STRUCTURE_TYPE_PIPELINE_COLOR_BLEND_STATE_CREATE_INFO = 26
VK_STRUCTURE_TYPE_PIPELINE_DYNAMIC_STATE_CREATE_INFO = 27
VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_CREATE_INFO = 28
VK_STRUCTURE_TYPE_COMPUTE_PIPELINE_CREATE_INFO = 29
VK_STRUCTURE_TYPE_PIPELINE_LAYOUT_CREATE_INFO = 30
VK_STRUCTURE_TYPE_SAMPLER_CREATE_INFO = 31
VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_CREATE_INFO = 32
VK_STRUCTURE_TYPE_DESCRIPTOR_POOL_CREATE_INFO = 33
VK_STRUCTURE_TYPE_DESCRIPTOR_SET_ALLOCATE_INFO = 34
VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET = 35
VK_STRUCTURE_TYPE_COPY_DESCRIPTOR_SET = 36
VK_STRUCTURE_TYPE_FRAMEBUFFER_CREATE_INFO = 37
VK_STRUCTURE_TYPE_RENDER_PASS_CREATE_INFO = 38
VK_STRUCTURE_TYPE_COMMAND_POOL_CREATE_INFO = 39
VK_STRUCTURE_TYPE_COMMAND_BUFFER_ALLOCATE_INFO = 40
VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_INFO = 41
VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO = 42
VK_STRUCTURE_TYPE_RENDER_PASS_BEGIN_INFO = 43
VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER = 44
VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER = 45
VK_STRUCTURE_TYPE_MEMORY_BARRIER = 46
VK_STRUCTURE_TYPE_LOADER_INSTANCE_CREATE_INFO = 47
VK_STRUCTURE_TYPE_LOADER_DEVICE_CREATE_INFO = 48
VK_STRUCTURE_TYPE_SWAPCHAIN_CREATE_INFO_KHR = 1000001000
VK_STRUCTURE_TYPE_PRESENT_INFO_KHR = 1000001001
VK_STRUCTURE_TYPE_DISPLAY_MODE_CREATE_INFO_KHR = 1000002000
VK_STRUCTURE_TYPE_DISPLAY_SURFACE_CREATE_INFO_KHR = 1000002001
VK_STRUCTURE_TYPE_DISPLAY_PRESENT_INFO_KHR = 1000003000
VK_STRUCTURE_TYPE_XLIB_SURFACE_CREATE_INFO_KHR = 1000004000
VK_STRUCTURE_TYPE_XCB_SURFACE_CREATE_INFO_KHR = 1000005000
VK_STRUCTURE_TYPE_WAYLAND_SURFACE_CREATE_INFO_KHR = 1000006000
VK_STRUCTURE_TYPE_MIR_SURFACE_CREATE_INFO_KHR = 1000007000
VK_STRUCTURE_TYPE_ANDROID_SURFACE_CREATE_INFO_KHR = 1000008000
VK_STRUCTURE_TYPE_WIN32_SURFACE_CREATE_INFO_KHR = 1000009000
VK_STRUCTURE_TYPE_DEBUG_REPORT_CALLBACK_CREATE_INFO_EXT = 1000011000
VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_RASTERIZATION_ORDER_AMD = 1000018000
VK_STRUCTURE_TYPE_DEBUG_MARKER_OBJECT_NAME_INFO_EXT = 1000022000
VK_STRUCTURE_TYPE_DEBUG_MARKER_OBJECT_TAG_INFO_EXT = 1000022001
VK_STRUCTURE_TYPE_DEBUG_MARKER_MARKER_INFO_EXT = 1000022002
VK_STRUCTURE_TYPE_DEDICATED_ALLOCATION_IMAGE_CREATE_INFO_NV = 1000026000
VK_STRUCTURE_TYPE_DEDICATED_ALLOCATION_BUFFER_CREATE_INFO_NV = 1000026001
VK_STRUCTURE_TYPE_DEDICATED_ALLOCATION_MEMORY_ALLOCATE_INFO_NV = 1000026002
VK_STRUCTURE_TYPE_EXTERNAL_MEMORY_IMAGE_CREATE_INFO_NV = 1000056000
VK_STRUCTURE_TYPE_EXPORT_MEMORY_ALLOCATE_INFO_NV = 1000056001
VK_STRUCTURE_TYPE_IMPORT_MEMORY_WIN32_HANDLE_INFO_NV = 1000057000
VK_STRUCTURE_TYPE_EXPORT_MEMORY_WIN32_HANDLE_INFO_NV = 1000057001
VK_STRUCTURE_TYPE_WIN32_KEYED_MUTEX_ACQUIRE_RELEASE_INFO_NV = 1000058000
VK_STRUCTURE_TYPE_VALIDATION_FLAGS_EXT = 1000061000
VK_SYSTEM_ALLOCATION_SCOPE_COMMAND = 0
VK_SYSTEM_ALLOCATION_SCOPE_OBJECT = 1
VK_SYSTEM_ALLOCATION_SCOPE_CACHE = 2
VK_SYSTEM_ALLOCATION_SCOPE_DEVICE = 3
VK_SYSTEM_ALLOCATION_SCOPE_INSTANCE = 4
VK_INTERNAL_ALLOCATION_TYPE_EXECUTABLE = 0
VK_SAMPLER_ADDRESS_MODE_REPEAT = 0
VK_SAMPLER_ADDRESS_MODE_MIRRORED_REPEAT = 1
VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE = 2
VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER = 3
VK_SAMPLER_ADDRESS_MODE_MIRROR_CLAMP_TO_EDGE = 4
VK_FILTER_NEAREST = 0
VK_FILTER_LINEAR = 1
VK_FILTER_CUBIC_IMG = 1000015000
VK_SAMPLER_MIPMAP_MODE_NEAREST = 0
VK_SAMPLER_MIPMAP_MODE_LINEAR = 1
VK_VERTEX_INPUT_RATE_VERTEX = 0
VK_VERTEX_INPUT_RATE_INSTANCE = 1
VK_PIPELINE_STAGE_TOP_OF_PIPE_BIT = 1
VK_PIPELINE_STAGE_DRAW_INDIRECT_BIT = 2
VK_PIPELINE_STAGE_VERTEX_INPUT_BIT = 4
VK_PIPELINE_STAGE_VERTEX_SHADER_BIT = 8
VK_PIPELINE_STAGE_TESSELLATION_CONTROL_SHADER_BIT = 16
VK_PIPELINE_STAGE_TESSELLATION_EVALUATION_SHADER_BIT = 32
VK_PIPELINE_STAGE_GEOMETRY_SHADER_BIT = 64
VK_PIPELINE_STAGE_FRAGMENT_SHADER_BIT = 128
VK_PIPELINE_STAGE_EARLY_FRAGMENT_TESTS_BIT = 256
VK_PIPELINE_STAGE_LATE_FRAGMENT_TESTS_BIT = 512
VK_PIPELINE_STAGE_COLOR_ATTACHMENT_OUTPUT_BIT = 1024
VK_PIPELINE_STAGE_COMPUTE_SHADER_BIT = 2048
VK_PIPELINE_STAGE_TRANSFER_BIT = 4096
VK_PIPELINE_STAGE_BOTTOM_OF_PIPE_BIT = 8192
VK_PIPELINE_STAGE_HOST_BIT = 16384
VK_PIPELINE_STAGE_ALL_GRAPHICS_BIT = 32768
VK_PIPELINE_STAGE_ALL_COMMANDS_BIT = 65536
VK_SPARSE_IMAGE_FORMAT_SINGLE_MIPTAIL_BIT = 1
VK_SPARSE_IMAGE_FORMAT_ALIGNED_MIP_SIZE_BIT = 2
VK_SPARSE_IMAGE_FORMAT_NONSTANDARD_BLOCK_SIZE_BIT = 4
VK_SAMPLE_COUNT_1_BIT = 1
VK_SAMPLE_COUNT_2_BIT = 2
VK_SAMPLE_COUNT_4_BIT = 4
VK_SAMPLE_COUNT_8_BIT = 8
VK_SAMPLE_COUNT_16_BIT = 16
VK_SAMPLE_COUNT_32_BIT = 32
VK_SAMPLE_COUNT_64_BIT = 64
VK_ATTACHMENT_DESCRIPTION_MAY_ALIAS_BIT = 1
VK_DESCRIPTOR_POOL_CREATE_FREE_DESCRIPTOR_SET_BIT = 1
VK_DEPENDENCY_BY_REGION_BIT = 1
VK_COLOR_SPACE_SRGB_NONLINEAR_KHR = 0
VK_COMPOSITE_ALPHA_OPAQUE_BIT_KHR = 1
VK_COMPOSITE_ALPHA_PRE_MULTIPLIED_BIT_KHR = 2
VK_COMPOSITE_ALPHA_POST_MULTIPLIED_BIT_KHR = 4
VK_COMPOSITE_ALPHA_INHERIT_BIT_KHR = 8
VK_COMPOSITE_ALPHA_FLAG_BITS_MAX_ENUM_KHR = 2147483647
VK_DISPLAY_PLANE_ALPHA_OPAQUE_BIT_KHR = 1
VK_DISPLAY_PLANE_ALPHA_GLOBAL_BIT_KHR = 2
VK_DISPLAY_PLANE_ALPHA_PER_PIXEL_BIT_KHR = 4
VK_DISPLAY_PLANE_ALPHA_PER_PIXEL_PREMULTIPLIED_BIT_KHR = 8
VK_PRESENT_MODE_IMMEDIATE_KHR = 0
VK_PRESENT_MODE_MAILBOX_KHR = 1
VK_PRESENT_MODE_FIFO_KHR = 2
VK_PRESENT_MODE_FIFO_RELAXED_KHR = 3
VK_SURFACE_TRANSFORM_IDENTITY_BIT_KHR = 1
VK_SURFACE_TRANSFORM_ROTATE_90_BIT_KHR = 2
VK_SURFACE_TRANSFORM_ROTATE_180_BIT_KHR = 4
VK_SURFACE_TRANSFORM_ROTATE_270_BIT_KHR = 8
VK_SURFACE_TRANSFORM_HORIZONTAL_MIRROR_BIT_KHR = 16
VK_SURFACE_TRANSFORM_HORIZONTAL_MIRROR_ROTATE_90_BIT_KHR = 32
VK_SURFACE_TRANSFORM_HORIZONTAL_MIRROR_ROTATE_180_BIT_KHR = 64
VK_SURFACE_TRANSFORM_HORIZONTAL_MIRROR_ROTATE_270_BIT_KHR = 128
VK_SURFACE_TRANSFORM_INHERIT_BIT_KHR = 256
VK_DEBUG_REPORT_INFORMATION_BIT_EXT = 1
VK_DEBUG_REPORT_WARNING_BIT_EXT = 2
VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT = 4
VK_DEBUG_REPORT_ERROR_BIT_EXT = 8
VK_DEBUG_REPORT_DEBUG_BIT_EXT = 16
VK_DEBUG_REPORT_OBJECT_TYPE_UNKNOWN_EXT = 0
VK_DEBUG_REPORT_OBJECT_TYPE_INSTANCE_EXT = 1
VK_DEBUG_REPORT_OBJECT_TYPE_PHYSICAL_DEVICE_EXT = 2
VK_DEBUG_REPORT_OBJECT_TYPE_DEVICE_EXT = 3
VK_DEBUG_REPORT_OBJECT_TYPE_QUEUE_EXT = 4
VK_DEBUG_REPORT_OBJECT_TYPE_SEMAPHORE_EXT = 5
VK_DEBUG_REPORT_OBJECT_TYPE_COMMAND_BUFFER_EXT = 6
VK_DEBUG_REPORT_OBJECT_TYPE_FENCE_EXT = 7
VK_DEBUG_REPORT_OBJECT_TYPE_DEVICE_MEMORY_EXT = 8
VK_DEBUG_REPORT_OBJECT_TYPE_BUFFER_EXT = 9
VK_DEBUG_REPORT_OBJECT_TYPE_IMAGE_EXT = 10
VK_DEBUG_REPORT_OBJECT_TYPE_EVENT_EXT = 11
VK_DEBUG_REPORT_OBJECT_TYPE_QUERY_POOL_EXT = 12
VK_DEBUG_REPORT_OBJECT_TYPE_BUFFER_VIEW_EXT = 13
VK_DEBUG_REPORT_OBJECT_TYPE_IMAGE_VIEW_EXT = 14
VK_DEBUG_REPORT_OBJECT_TYPE_SHADER_MODULE_EXT = 15
VK_DEBUG_REPORT_OBJECT_TYPE_PIPELINE_CACHE_EXT = 16
VK_DEBUG_REPORT_OBJECT_TYPE_PIPELINE_LAYOUT_EXT = 17
VK_DEBUG_REPORT_OBJECT_TYPE_RENDER_PASS_EXT = 18
VK_DEBUG_REPORT_OBJECT_TYPE_PIPELINE_EXT = 19
VK_DEBUG_REPORT_OBJECT_TYPE_DESCRIPTOR_SET_LAYOUT_EXT = 20
VK_DEBUG_REPORT_OBJECT_TYPE_SAMPLER_EXT = 21
VK_DEBUG_REPORT_OBJECT_TYPE_DESCRIPTOR_POOL_EXT = 22
VK_DEBUG_REPORT_OBJECT_TYPE_DESCRIPTOR_SET_EXT = 23
VK_DEBUG_REPORT_OBJECT_TYPE_FRAMEBUFFER_EXT = 24
VK_DEBUG_REPORT_OBJECT_TYPE_COMMAND_POOL_EXT = 25
VK_DEBUG_REPORT_OBJECT_TYPE_SURFACE_KHR_EXT = 26
VK_DEBUG_REPORT_OBJECT_TYPE_SWAPCHAIN_KHR_EXT = 27
VK_DEBUG_REPORT_OBJECT_TYPE_DEBUG_REPORT_EXT = 28
VK_DEBUG_REPORT_ERROR_NONE_EXT = 0
VK_DEBUG_REPORT_ERROR_CALLBACK_REF_EXT = 1
VK_RASTERIZATION_ORDER_STRICT_AMD = 0
VK_RASTERIZATION_ORDER_RELAXED_AMD = 1
VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_BIT_NV = 1
VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_KMT_BIT_NV = 2
VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_IMAGE_BIT_NV = 4
VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_IMAGE_KMT_BIT_NV = 8
VK_EXTERNAL_MEMORY_FEATURE_DEDICATED_ONLY_BIT_NV = 1
VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT_NV = 2
VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT_NV = 4
VK_VALIDATION_CHECK_ALL_EXT = 0

def VK_MAKE_VERSION(major, minor, patch):
	return (((major) << 22) | ((minor) << 12) | (patch))

def VK_VERSION_MAJOR(version):
	return version>>22

def VK_VERSION_MINOR(version):
	return (version>>12)&0x3ff

def VK_VERSION_PATCH(version):
	return version&0xfff

VK_API_VERSION = VK_MAKE_VERSION(1, 0, 0)
VK_API_VERSION_1_0 = VK_MAKE_VERSION(1, 0, 0)

VK_NULL_HANDLE = 0

VK_MAX_PHYSICAL_DEVICE_NAME_SIZE = 256
VK_UUID_SIZE = 16
VK_MAX_EXTENSION_NAME_SIZE = 256
VK_MAX_DESCRIPTION_SIZE = 256
VK_MAX_MEMORY_TYPES = 32
VK_MAX_MEMORY_HEAPS = 16
VK_LOD_CLAMP_NONE = 1000.0
VK_REMAINING_MIP_LEVELS = -1
VK_REMAINING_ARRAY_LAYERS = -1
VK_WHOLE_SIZE = -1
VK_ATTACHMENT_UNUSED = -1
VK_TRUE = 1
VK_FALSE = 0
VK_QUEUE_FAMILY_IGNORED = -1
VK_SUBPASS_EXTERNAL = -1
VK_KHR_surface = 1
VK_KHR_SURFACE_SPEC_VERSION = 25
VK_KHR_SURFACE_EXTENSION_NAME = "VK_KHR_surface"
VK_COLORSPACE_SRGB_NONLINEAR_KHR = VK_COLOR_SPACE_SRGB_NONLINEAR_KHR
VK_KHR_swapchain = 1
VK_KHR_SWAPCHAIN_SPEC_VERSION = 68
VK_KHR_SWAPCHAIN_EXTENSION_NAME = "VK_KHR_swapchain"
VK_KHR_display = 1
VK_KHR_DISPLAY_SPEC_VERSION = 21
VK_KHR_DISPLAY_EXTENSION_NAME = "VK_KHR_display"
VK_KHR_display_swapchain = 1
VK_KHR_DISPLAY_SWAPCHAIN_SPEC_VERSION = 9
VK_KHR_DISPLAY_SWAPCHAIN_EXTENSION_NAME = "VK_KHR_display_swapchain"
VK_KHR_xlib_surface = 1
VK_KHR_XLIB_SURFACE_SPEC_VERSION = 6
VK_KHR_XLIB_SURFACE_EXTENSION_NAME = "VK_KHR_xlib_surface"
VK_KHR_xcb_surface = 1
VK_KHR_XCB_SURFACE_SPEC_VERSION = 6
VK_KHR_XCB_SURFACE_EXTENSION_NAME = "VK_KHR_xcb_surface"
VK_KHR_wayland_surface = 1
VK_KHR_WAYLAND_SURFACE_SPEC_VERSION = 5
VK_KHR_WAYLAND_SURFACE_EXTENSION_NAME = "VK_KHR_wayland_surface"
VK_KHR_mir_surface = 1
VK_KHR_MIR_SURFACE_SPEC_VERSION = 4
VK_KHR_MIR_SURFACE_EXTENSION_NAME = "VK_KHR_mir_surface"
VK_KHR_android_surface = 1
VK_KHR_ANDROID_SURFACE_SPEC_VERSION = 6
VK_KHR_ANDROID_SURFACE_EXTENSION_NAME = "VK_KHR_android_surface"
VK_KHR_win32_surface = 1
VK_KHR_WIN32_SURFACE_SPEC_VERSION = 5
VK_KHR_WIN32_SURFACE_EXTENSION_NAME = "VK_KHR_win32_surface"
VK_EXT_debug_report = 1
VK_EXT_DEBUG_REPORT_SPEC_VERSION = 3
VK_EXT_DEBUG_REPORT_EXTENSION_NAME = "VK_EXT_debug_report"
VK_STRUCTURE_TYPE_DEBUG_REPORT_CREATE_INFO_EXT = VK_STRUCTURE_TYPE_DEBUG_REPORT_CALLBACK_CREATE_INFO_EXT
VK_NV_glsl_shader = 1
VK_NV_GLSL_SHADER_SPEC_VERSION = 1
VK_NV_GLSL_SHADER_EXTENSION_NAME = "VK_NV_glsl_shader"
VK_KHR_sampler_mirror_clamp_to_edge = 1
VK_KHR_SAMPLER_MIRROR_CLAMP_TO_EDGE_SPEC_VERSION = 1
VK_KHR_SAMPLER_MIRROR_CLAMP_TO_EDGE_EXTENSION_NAME = "VK_KHR_sampler_mirror_clamp_to_edge"
VK_IMG_filter_cubic = 1
VK_IMG_FILTER_CUBIC_SPEC_VERSION = 1
VK_IMG_FILTER_CUBIC_EXTENSION_NAME = "VK_IMG_filter_cubic"
VK_AMD_rasterization_order = 1
VK_AMD_RASTERIZATION_ORDER_SPEC_VERSION = 1
VK_AMD_RASTERIZATION_ORDER_EXTENSION_NAME = "VK_AMD_rasterization_order"
VK_AMD_shader_trinary_minmax = 1
VK_AMD_SHADER_TRINARY_MINMAX_SPEC_VERSION = 1
VK_AMD_SHADER_TRINARY_MINMAX_EXTENSION_NAME = "VK_AMD_shader_trinary_minmax"
VK_AMD_shader_explicit_vertex_parameter = 1
VK_AMD_SHADER_EXPLICIT_VERTEX_PARAMETER_SPEC_VERSION = 1
VK_AMD_SHADER_EXPLICIT_VERTEX_PARAMETER_EXTENSION_NAME = "VK_AMD_shader_explicit_vertex_parameter"
VK_EXT_debug_marker = 1
VK_EXT_DEBUG_MARKER_SPEC_VERSION = 3
VK_EXT_DEBUG_MARKER_EXTENSION_NAME = "VK_EXT_debug_marker"
VK_AMD_gcn_shader = 1
VK_AMD_GCN_SHADER_SPEC_VERSION = 1
VK_AMD_GCN_SHADER_EXTENSION_NAME = "VK_AMD_gcn_shader"
VK_NV_dedicated_allocation = 1
VK_NV_DEDICATED_ALLOCATION_SPEC_VERSION = 1
VK_NV_DEDICATED_ALLOCATION_EXTENSION_NAME = "VK_NV_dedicated_allocation"
VK_AMD_draw_indirect_count = 1
VK_AMD_DRAW_INDIRECT_COUNT_SPEC_VERSION = 1
VK_AMD_DRAW_INDIRECT_COUNT_EXTENSION_NAME = "VK_AMD_draw_indirect_count"
VK_AMD_negative_viewport_height = 1
VK_AMD_NEGATIVE_VIEWPORT_HEIGHT_SPEC_VERSION = 0
VK_AMD_NEGATIVE_VIEWPORT_HEIGHT_EXTENSION_NAME = "VK_AMD_negative_viewport_height"
VK_AMD_gpu_shader_half_float = 1
VK_AMD_GPU_SHADER_HALF_FLOAT_SPEC_VERSION = 1
VK_AMD_GPU_SHADER_HALF_FLOAT_EXTENSION_NAME = "VK_AMD_gpu_shader_half_float"
VK_AMD_shader_ballot = 1
VK_AMD_SHADER_BALLOT_SPEC_VERSION = 0
VK_AMD_SHADER_BALLOT_EXTENSION_NAME = "VK_AMD_shader_ballot"
VK_IMG_format_pvrtc = 1
VK_IMG_FORMAT_PVRTC_SPEC_VERSION = 1
VK_IMG_FORMAT_PVRTC_EXTENSION_NAME = "VK_IMG_format_pvrtc"
VK_NV_external_memory_capabilities = 1
VK_NV_EXTERNAL_MEMORY_CAPABILITIES_SPEC_VERSION = 1
VK_NV_EXTERNAL_MEMORY_CAPABILITIES_EXTENSION_NAME = "VK_NV_external_memory_capabilities"
VK_NV_external_memory = 1
VK_NV_EXTERNAL_MEMORY_SPEC_VERSION = 1
VK_NV_EXTERNAL_MEMORY_EXTENSION_NAME = "VK_NV_external_memory"
VK_NV_external_memory_win32 = 1
VK_NV_EXTERNAL_MEMORY_WIN32_SPEC_VERSION = 1
VK_NV_EXTERNAL_MEMORY_WIN32_EXTENSION_NAME = "VK_NV_external_memory_win32"
VK_NV_win32_keyed_mutex = 1
VK_NV_WIN32_KEYED_MUTEX_SPEC_VERSION = 1
VK_NV_WIN32_KEYED_MUTEX_EXTENSION_NAME = "VK_NV_win32_keyed_mutex"
VK_EXT_validation_flags = 1
VK_EXT_VALIDATION_FLAGS_SPEC_VERSION = 1
VK_EXT_VALIDATION_FLAGS_EXTENSION_NAME = "VK_EXT_validation_flags"

class VkException(Exception):
	pass

class VkError(Exception):
	pass

class VkIncomplete(VkException):
	pass
class VkNotReady(VkException):
	pass
class VkTimeout(VkException):
	pass
class VkEventSet(VkException):
	pass
class VkEventReset(VkException):
	pass
class VkSuboptimalKHR(VkException):
	pass

class VkErrorOutOfHostMemory(VkError):
	pass
class VkErrorOutOfDeviceMemory(VkError):
	pass
class VkErrorInitializationFailed(VkError):
	pass
class VkErrorDeviceLost(VkError):
	pass
class VkErrorMemoryMapFailed(VkError):
	pass
class VkErrorLayerNotPresent(VkError):
	pass
class VkErrorExtensionNotPresent(VkError):
	pass
class VkErrorFeatureNotPresent(VkError):
	pass
class VkErrorIncompatibleDriver(VkError):
	pass
class VkErrorTooManyObjects(VkError):
	pass
class VkErrorFormatNotSupported(VkError):
	pass
class VkErrorFragmentedPool(VkError):
	pass
class VkErrorSurfaceLostKHR(VkError):
	pass
class VkErrorNativeWindowInUseKHR(VkError):
	pass
class VkErrorOutOfDateKHR(VkError):
	pass
class VkErrorIncompatibleDisplayKHR(VkError):
	pass
class VkErrorValidationFailedEXT(VkError):
	pass
class VkErrorInvalidShaderNV(VkError):
	pass

_exception_codes = {
	VK_INCOMPLETE:VkIncomplete,
	VK_NOT_READY:VkNotReady,
	VK_TIMEOUT:VkTimeout,
	VK_EVENT_SET:VkEventSet,
	VK_EVENT_RESET:VkEventReset,
	VK_SUBOPTIMAL_KHR:VkSuboptimalKHR,
	VK_ERROR_OUT_OF_HOST_MEMORY:VkErrorOutOfHostMemory,
	VK_ERROR_OUT_OF_DEVICE_MEMORY:VkErrorOutOfDeviceMemory,
	VK_ERROR_INITIALIZATION_FAILED:VkErrorInitializationFailed,
	VK_ERROR_DEVICE_LOST:VkErrorDeviceLost,
	VK_ERROR_MEMORY_MAP_FAILED:VkErrorMemoryMapFailed,
	VK_ERROR_LAYER_NOT_PRESENT:VkErrorLayerNotPresent,
	VK_ERROR_EXTENSION_NOT_PRESENT:VkErrorExtensionNotPresent,
	VK_ERROR_FEATURE_NOT_PRESENT:VkErrorFeatureNotPresent,
	VK_ERROR_INCOMPATIBLE_DRIVER:VkErrorIncompatibleDriver,
	VK_ERROR_TOO_MANY_OBJECTS:VkErrorTooManyObjects,
	VK_ERROR_FORMAT_NOT_SUPPORTED:VkErrorFormatNotSupported,
	VK_ERROR_FRAGMENTED_POOL:VkErrorFragmentedPool,
	VK_ERROR_SURFACE_LOST_KHR:VkErrorSurfaceLostKHR,
	VK_ERROR_NATIVE_WINDOW_IN_USE_KHR:VkErrorNativeWindowInUseKHR,
	VK_ERROR_OUT_OF_DATE_KHR:VkErrorOutOfDateKHR,
	VK_ERROR_INCOMPATIBLE_DISPLAY_KHR:VkErrorIncompatibleDisplayKHR,
	VK_ERROR_VALIDATION_FAILED_EXT:VkErrorValidationFailedEXT,
	VK_ERROR_INVALID_SHADER_NV:VkErrorInvalidShaderNV
}

vkInternalAllocationNotification = ffi.callback('PFN_vkInternalAllocationNotification')
vkInternalFreeNotification = ffi.callback('PFN_vkInternalFreeNotification')
vkReallocationFunction = ffi.callback('PFN_vkReallocationFunction')
vkAllocationFunction = ffi.callback('PFN_vkAllocationFunction')
vkFreeFunction = ffi.callback('PFN_vkFreeFunction')
vkVoidFunction = ffi.callback('PFN_vkVoidFunction')
vkDebugReportCallbackEXT = ffi.callback('PFN_vkDebugReportCallbackEXT')

def _new(ctype, **kwargs):
	_type = ffi.typeof(ctype)
	kwargs = {k:kwargs[k] for k in kwargs if kwargs[k]}
	ptrs = {k:_castToPtr(kwargs[k], dict(_type.fields)[k].type) for k in kwargs if dict(_type.fields)[k].type.kind=='pointer'}
	ret = ffi.new(_type.cname+'*', dict(kwargs, **{k:v for k, (v, _) in ptrs.items()}))[0]
	_weakkey_dict[ret] = tuple(v for _, v in ptrs.values() if v!=ffi.NULL)
	return ret


def VkOffset2D(x=None, y=None):
	return _new('VkOffset2D', x=x, y=y)

def VkOffset3D(x=None, y=None, z=None):
	return _new('VkOffset3D', x=x, y=y, z=z)

def VkExtent2D(width=None, height=None):
	return _new('VkExtent2D', width=width, height=height)

def VkExtent3D(width=None, height=None, depth=None):
	return _new('VkExtent3D', width=width, height=height, depth=depth)

def VkViewport(x=None, y=None, width=None, height=None, minDepth=None, maxDepth=None):
	return _new('VkViewport', x=x, y=y, width=width, height=height, minDepth=minDepth, maxDepth=maxDepth)

def VkRect2D(offset=None, extent=None):
	return _new('VkRect2D', offset=offset, extent=extent)

def VkRect3D(offset=None, extent=None):
	return _new('VkRect3D', offset=offset, extent=extent)

def VkClearRect(rect=None, baseArrayLayer=None, layerCount=None):
	return _new('VkClearRect', rect=rect, baseArrayLayer=baseArrayLayer, layerCount=layerCount)

def VkComponentMapping(r=None, g=None, b=None, a=None):
	return _new('VkComponentMapping', r=r, g=g, b=b, a=a)

def VkPhysicalDeviceLimits(maxImageDimension1D=None, maxImageDimension2D=None, maxImageDimension3D=None, maxImageDimensionCube=None, maxImageArrayLayers=None, maxTexelBufferElements=None, maxUniformBufferRange=None, maxStorageBufferRange=None, maxPushConstantsSize=None, maxMemoryAllocationCount=None, maxSamplerAllocationCount=None, bufferImageGranularity=None, sparseAddressSpaceSize=None, maxBoundDescriptorSets=None, maxPerStageDescriptorSamplers=None, maxPerStageDescriptorUniformBuffers=None, maxPerStageDescriptorStorageBuffers=None, maxPerStageDescriptorSampledImages=None, maxPerStageDescriptorStorageImages=None, maxPerStageDescriptorInputAttachments=None, maxPerStageResources=None, maxDescriptorSetSamplers=None, maxDescriptorSetUniformBuffers=None, maxDescriptorSetUniformBuffersDynamic=None, maxDescriptorSetStorageBuffers=None, maxDescriptorSetStorageBuffersDynamic=None, maxDescriptorSetSampledImages=None, maxDescriptorSetStorageImages=None, maxDescriptorSetInputAttachments=None, maxVertexInputAttributes=None, maxVertexInputBindings=None, maxVertexInputAttributeOffset=None, maxVertexInputBindingStride=None, maxVertexOutputComponents=None, maxTessellationGenerationLevel=None, maxTessellationPatchSize=None, maxTessellationControlPerVertexInputComponents=None, maxTessellationControlPerVertexOutputComponents=None, maxTessellationControlPerPatchOutputComponents=None, maxTessellationControlTotalOutputComponents=None, maxTessellationEvaluationInputComponents=None, maxTessellationEvaluationOutputComponents=None, maxGeometryShaderInvocations=None, maxGeometryInputComponents=None, maxGeometryOutputComponents=None, maxGeometryOutputVertices=None, maxGeometryTotalOutputComponents=None, maxFragmentInputComponents=None, maxFragmentOutputAttachments=None, maxFragmentDualSrcAttachments=None, maxFragmentCombinedOutputResources=None, maxComputeSharedMemorySize=None, maxComputeWorkGroupCount=None, maxComputeWorkGroupInvocations=None, maxComputeWorkGroupSize=None, subPixelPrecisionBits=None, subTexelPrecisionBits=None, mipmapPrecisionBits=None, maxDrawIndexedIndexValue=None, maxDrawIndirectCount=None, maxSamplerLodBias=None, maxSamplerAnisotropy=None, maxViewports=None, maxViewportDimensions=None, viewportBoundsRange=None, viewportSubPixelBits=None, minMemoryMapAlignment=None, minTexelBufferOffsetAlignment=None, minUniformBufferOffsetAlignment=None, minStorageBufferOffsetAlignment=None, minTexelOffset=None, maxTexelOffset=None, minTexelGatherOffset=None, maxTexelGatherOffset=None, minInterpolationOffset=None, maxInterpolationOffset=None, subPixelInterpolationOffsetBits=None, maxFramebufferWidth=None, maxFramebufferHeight=None, maxFramebufferLayers=None, framebufferColorSampleCounts=None, framebufferDepthSampleCounts=None, framebufferStencilSampleCounts=None, framebufferNoAttachmentsSampleCounts=None, maxColorAttachments=None, sampledImageColorSampleCounts=None, sampledImageIntegerSampleCounts=None, sampledImageDepthSampleCounts=None, sampledImageStencilSampleCounts=None, storageImageSampleCounts=None, maxSampleMaskWords=None, timestampComputeAndGraphics=None, timestampPeriod=None, maxClipDistances=None, maxCullDistances=None, maxCombinedClipAndCullDistances=None, discreteQueuePriorities=None, pointSizeRange=None, lineWidthRange=None, pointSizeGranularity=None, lineWidthGranularity=None, strictLines=None, standardSampleLocations=None, optimalBufferCopyOffsetAlignment=None, optimalBufferCopyRowPitchAlignment=None, nonCoherentAtomSize=None):
	return _new('VkPhysicalDeviceLimits', maxImageDimension1D=maxImageDimension1D, maxImageDimension2D=maxImageDimension2D, maxImageDimension3D=maxImageDimension3D, maxImageDimensionCube=maxImageDimensionCube, maxImageArrayLayers=maxImageArrayLayers, maxTexelBufferElements=maxTexelBufferElements, maxUniformBufferRange=maxUniformBufferRange, maxStorageBufferRange=maxStorageBufferRange, maxPushConstantsSize=maxPushConstantsSize, maxMemoryAllocationCount=maxMemoryAllocationCount, maxSamplerAllocationCount=maxSamplerAllocationCount, bufferImageGranularity=bufferImageGranularity, sparseAddressSpaceSize=sparseAddressSpaceSize, maxBoundDescriptorSets=maxBoundDescriptorSets, maxPerStageDescriptorSamplers=maxPerStageDescriptorSamplers, maxPerStageDescriptorUniformBuffers=maxPerStageDescriptorUniformBuffers, maxPerStageDescriptorStorageBuffers=maxPerStageDescriptorStorageBuffers, maxPerStageDescriptorSampledImages=maxPerStageDescriptorSampledImages, maxPerStageDescriptorStorageImages=maxPerStageDescriptorStorageImages, maxPerStageDescriptorInputAttachments=maxPerStageDescriptorInputAttachments, maxPerStageResources=maxPerStageResources, maxDescriptorSetSamplers=maxDescriptorSetSamplers, maxDescriptorSetUniformBuffers=maxDescriptorSetUniformBuffers, maxDescriptorSetUniformBuffersDynamic=maxDescriptorSetUniformBuffersDynamic, maxDescriptorSetStorageBuffers=maxDescriptorSetStorageBuffers, maxDescriptorSetStorageBuffersDynamic=maxDescriptorSetStorageBuffersDynamic, maxDescriptorSetSampledImages=maxDescriptorSetSampledImages, maxDescriptorSetStorageImages=maxDescriptorSetStorageImages, maxDescriptorSetInputAttachments=maxDescriptorSetInputAttachments, maxVertexInputAttributes=maxVertexInputAttributes, maxVertexInputBindings=maxVertexInputBindings, maxVertexInputAttributeOffset=maxVertexInputAttributeOffset, maxVertexInputBindingStride=maxVertexInputBindingStride, maxVertexOutputComponents=maxVertexOutputComponents, maxTessellationGenerationLevel=maxTessellationGenerationLevel, maxTessellationPatchSize=maxTessellationPatchSize, maxTessellationControlPerVertexInputComponents=maxTessellationControlPerVertexInputComponents, maxTessellationControlPerVertexOutputComponents=maxTessellationControlPerVertexOutputComponents, maxTessellationControlPerPatchOutputComponents=maxTessellationControlPerPatchOutputComponents, maxTessellationControlTotalOutputComponents=maxTessellationControlTotalOutputComponents, maxTessellationEvaluationInputComponents=maxTessellationEvaluationInputComponents, maxTessellationEvaluationOutputComponents=maxTessellationEvaluationOutputComponents, maxGeometryShaderInvocations=maxGeometryShaderInvocations, maxGeometryInputComponents=maxGeometryInputComponents, maxGeometryOutputComponents=maxGeometryOutputComponents, maxGeometryOutputVertices=maxGeometryOutputVertices, maxGeometryTotalOutputComponents=maxGeometryTotalOutputComponents, maxFragmentInputComponents=maxFragmentInputComponents, maxFragmentOutputAttachments=maxFragmentOutputAttachments, maxFragmentDualSrcAttachments=maxFragmentDualSrcAttachments, maxFragmentCombinedOutputResources=maxFragmentCombinedOutputResources, maxComputeSharedMemorySize=maxComputeSharedMemorySize, maxComputeWorkGroupCount=maxComputeWorkGroupCount, maxComputeWorkGroupInvocations=maxComputeWorkGroupInvocations, maxComputeWorkGroupSize=maxComputeWorkGroupSize, subPixelPrecisionBits=subPixelPrecisionBits, subTexelPrecisionBits=subTexelPrecisionBits, mipmapPrecisionBits=mipmapPrecisionBits, maxDrawIndexedIndexValue=maxDrawIndexedIndexValue, maxDrawIndirectCount=maxDrawIndirectCount, maxSamplerLodBias=maxSamplerLodBias, maxSamplerAnisotropy=maxSamplerAnisotropy, maxViewports=maxViewports, maxViewportDimensions=maxViewportDimensions, viewportBoundsRange=viewportBoundsRange, viewportSubPixelBits=viewportSubPixelBits, minMemoryMapAlignment=minMemoryMapAlignment, minTexelBufferOffsetAlignment=minTexelBufferOffsetAlignment, minUniformBufferOffsetAlignment=minUniformBufferOffsetAlignment, minStorageBufferOffsetAlignment=minStorageBufferOffsetAlignment, minTexelOffset=minTexelOffset, maxTexelOffset=maxTexelOffset, minTexelGatherOffset=minTexelGatherOffset, maxTexelGatherOffset=maxTexelGatherOffset, minInterpolationOffset=minInterpolationOffset, maxInterpolationOffset=maxInterpolationOffset, subPixelInterpolationOffsetBits=subPixelInterpolationOffsetBits, maxFramebufferWidth=maxFramebufferWidth, maxFramebufferHeight=maxFramebufferHeight, maxFramebufferLayers=maxFramebufferLayers, framebufferColorSampleCounts=framebufferColorSampleCounts, framebufferDepthSampleCounts=framebufferDepthSampleCounts, framebufferStencilSampleCounts=framebufferStencilSampleCounts, framebufferNoAttachmentsSampleCounts=framebufferNoAttachmentsSampleCounts, maxColorAttachments=maxColorAttachments, sampledImageColorSampleCounts=sampledImageColorSampleCounts, sampledImageIntegerSampleCounts=sampledImageIntegerSampleCounts, sampledImageDepthSampleCounts=sampledImageDepthSampleCounts, sampledImageStencilSampleCounts=sampledImageStencilSampleCounts, storageImageSampleCounts=storageImageSampleCounts, maxSampleMaskWords=maxSampleMaskWords, timestampComputeAndGraphics=timestampComputeAndGraphics, timestampPeriod=timestampPeriod, maxClipDistances=maxClipDistances, maxCullDistances=maxCullDistances, maxCombinedClipAndCullDistances=maxCombinedClipAndCullDistances, discreteQueuePriorities=discreteQueuePriorities, pointSizeRange=pointSizeRange, lineWidthRange=lineWidthRange, pointSizeGranularity=pointSizeGranularity, lineWidthGranularity=lineWidthGranularity, strictLines=strictLines, standardSampleLocations=standardSampleLocations, optimalBufferCopyOffsetAlignment=optimalBufferCopyOffsetAlignment, optimalBufferCopyRowPitchAlignment=optimalBufferCopyRowPitchAlignment, nonCoherentAtomSize=nonCoherentAtomSize)

def VkPhysicalDeviceSparseProperties(residencyStandard2DBlockShape=None, residencyStandard2DMultisampleBlockShape=None, residencyStandard3DBlockShape=None, residencyAlignedMipSize=None, residencyNonResidentStrict=None):
	return _new('VkPhysicalDeviceSparseProperties', residencyStandard2DBlockShape=residencyStandard2DBlockShape, residencyStandard2DMultisampleBlockShape=residencyStandard2DMultisampleBlockShape, residencyStandard3DBlockShape=residencyStandard3DBlockShape, residencyAlignedMipSize=residencyAlignedMipSize, residencyNonResidentStrict=residencyNonResidentStrict)

def VkPhysicalDeviceProperties(apiVersion=None, driverVersion=None, vendorID=None, deviceID=None, deviceType=None, deviceName=None, pipelineCacheUUID=None, limits=None, sparseProperties=None):
	return _new('VkPhysicalDeviceProperties', apiVersion=apiVersion, driverVersion=driverVersion, vendorID=vendorID, deviceID=deviceID, deviceType=deviceType, deviceName=deviceName, pipelineCacheUUID=pipelineCacheUUID, limits=limits, sparseProperties=sparseProperties)

def VkExtensionProperties(extensionName=None, specVersion=None):
	return _new('VkExtensionProperties', extensionName=extensionName, specVersion=specVersion)

def VkLayerProperties(layerName=None, specVersion=None, implementationVersion=None, description=None):
	return _new('VkLayerProperties', layerName=layerName, specVersion=specVersion, implementationVersion=implementationVersion, description=description)

def VkApplicationInfo(sType=VK_STRUCTURE_TYPE_APPLICATION_INFO, pNext=None, pApplicationName=None, applicationVersion=None, pEngineName=None, engineVersion=None, apiVersion=None):
	return _new('VkApplicationInfo', sType=sType, pNext=pNext, pApplicationName=pApplicationName, applicationVersion=applicationVersion, pEngineName=pEngineName, engineVersion=engineVersion, apiVersion=apiVersion)

def VkAllocationCallbacks(pUserData=None, pfnAllocation=None, pfnReallocation=None, pfnFree=None, pfnInternalAllocation=None, pfnInternalFree=None):
	return _new('VkAllocationCallbacks', pUserData=pUserData, pfnAllocation=pfnAllocation, pfnReallocation=pfnReallocation, pfnFree=pfnFree, pfnInternalAllocation=pfnInternalAllocation, pfnInternalFree=pfnInternalFree)

def VkDeviceQueueCreateInfo(sType=VK_STRUCTURE_TYPE_DEVICE_QUEUE_CREATE_INFO, pNext=None, flags=None, queueFamilyIndex=None, queueCount=None, pQueuePriorities=[]):
	if queueCount is None:
		queueCount = len(pQueuePriorities)
	return _new('VkDeviceQueueCreateInfo', sType=sType, pNext=pNext, flags=flags, queueFamilyIndex=queueFamilyIndex, queueCount=queueCount, pQueuePriorities=pQueuePriorities)

def VkPhysicalDeviceFeatures(robustBufferAccess=None, fullDrawIndexUint32=None, imageCubeArray=None, independentBlend=None, geometryShader=None, tessellationShader=None, sampleRateShading=None, dualSrcBlend=None, logicOp=None, multiDrawIndirect=None, drawIndirectFirstInstance=None, depthClamp=None, depthBiasClamp=None, fillModeNonSolid=None, depthBounds=None, wideLines=None, largePoints=None, alphaToOne=None, multiViewport=None, samplerAnisotropy=None, textureCompressionETC2=None, textureCompressionASTC_LDR=None, textureCompressionBC=None, occlusionQueryPrecise=None, pipelineStatisticsQuery=None, vertexPipelineStoresAndAtomics=None, fragmentStoresAndAtomics=None, shaderTessellationAndGeometryPointSize=None, shaderImageGatherExtended=None, shaderStorageImageExtendedFormats=None, shaderStorageImageMultisample=None, shaderStorageImageReadWithoutFormat=None, shaderStorageImageWriteWithoutFormat=None, shaderUniformBufferArrayDynamicIndexing=None, shaderSampledImageArrayDynamicIndexing=None, shaderStorageBufferArrayDynamicIndexing=None, shaderStorageImageArrayDynamicIndexing=None, shaderClipDistance=None, shaderCullDistance=None, shaderFloat64=None, shaderInt64=None, shaderInt16=None, shaderResourceResidency=None, shaderResourceMinLod=None, sparseBinding=None, sparseResidencyBuffer=None, sparseResidencyImage2D=None, sparseResidencyImage3D=None, sparseResidency2Samples=None, sparseResidency4Samples=None, sparseResidency8Samples=None, sparseResidency16Samples=None, sparseResidencyAliased=None, variableMultisampleRate=None, inheritedQueries=None):
	return _new('VkPhysicalDeviceFeatures', robustBufferAccess=robustBufferAccess, fullDrawIndexUint32=fullDrawIndexUint32, imageCubeArray=imageCubeArray, independentBlend=independentBlend, geometryShader=geometryShader, tessellationShader=tessellationShader, sampleRateShading=sampleRateShading, dualSrcBlend=dualSrcBlend, logicOp=logicOp, multiDrawIndirect=multiDrawIndirect, drawIndirectFirstInstance=drawIndirectFirstInstance, depthClamp=depthClamp, depthBiasClamp=depthBiasClamp, fillModeNonSolid=fillModeNonSolid, depthBounds=depthBounds, wideLines=wideLines, largePoints=largePoints, alphaToOne=alphaToOne, multiViewport=multiViewport, samplerAnisotropy=samplerAnisotropy, textureCompressionETC2=textureCompressionETC2, textureCompressionASTC_LDR=textureCompressionASTC_LDR, textureCompressionBC=textureCompressionBC, occlusionQueryPrecise=occlusionQueryPrecise, pipelineStatisticsQuery=pipelineStatisticsQuery, vertexPipelineStoresAndAtomics=vertexPipelineStoresAndAtomics, fragmentStoresAndAtomics=fragmentStoresAndAtomics, shaderTessellationAndGeometryPointSize=shaderTessellationAndGeometryPointSize, shaderImageGatherExtended=shaderImageGatherExtended, shaderStorageImageExtendedFormats=shaderStorageImageExtendedFormats, shaderStorageImageMultisample=shaderStorageImageMultisample, shaderStorageImageReadWithoutFormat=shaderStorageImageReadWithoutFormat, shaderStorageImageWriteWithoutFormat=shaderStorageImageWriteWithoutFormat, shaderUniformBufferArrayDynamicIndexing=shaderUniformBufferArrayDynamicIndexing, shaderSampledImageArrayDynamicIndexing=shaderSampledImageArrayDynamicIndexing, shaderStorageBufferArrayDynamicIndexing=shaderStorageBufferArrayDynamicIndexing, shaderStorageImageArrayDynamicIndexing=shaderStorageImageArrayDynamicIndexing, shaderClipDistance=shaderClipDistance, shaderCullDistance=shaderCullDistance, shaderFloat64=shaderFloat64, shaderInt64=shaderInt64, shaderInt16=shaderInt16, shaderResourceResidency=shaderResourceResidency, shaderResourceMinLod=shaderResourceMinLod, sparseBinding=sparseBinding, sparseResidencyBuffer=sparseResidencyBuffer, sparseResidencyImage2D=sparseResidencyImage2D, sparseResidencyImage3D=sparseResidencyImage3D, sparseResidency2Samples=sparseResidency2Samples, sparseResidency4Samples=sparseResidency4Samples, sparseResidency8Samples=sparseResidency8Samples, sparseResidency16Samples=sparseResidency16Samples, sparseResidencyAliased=sparseResidencyAliased, variableMultisampleRate=variableMultisampleRate, inheritedQueries=inheritedQueries)

def VkDeviceCreateInfo(sType=VK_STRUCTURE_TYPE_DEVICE_CREATE_INFO, pNext=None, flags=None, queueCreateInfoCount=None, pQueueCreateInfos=[], enabledLayerCount=None, ppEnabledLayerNames=[], enabledExtensionCount=None, ppEnabledExtensionNames=[], pEnabledFeatures=None):
	if queueCreateInfoCount is None:
		queueCreateInfoCount = len(pQueueCreateInfos)
	if enabledLayerCount is None:
		enabledLayerCount = len(ppEnabledLayerNames)
	if enabledExtensionCount is None:
		enabledExtensionCount = len(ppEnabledExtensionNames)
	return _new('VkDeviceCreateInfo', sType=sType, pNext=pNext, flags=flags, queueCreateInfoCount=queueCreateInfoCount, pQueueCreateInfos=pQueueCreateInfos, enabledLayerCount=enabledLayerCount, ppEnabledLayerNames=ppEnabledLayerNames, enabledExtensionCount=enabledExtensionCount, ppEnabledExtensionNames=ppEnabledExtensionNames, pEnabledFeatures=pEnabledFeatures)

def VkInstanceCreateInfo(sType=VK_STRUCTURE_TYPE_INSTANCE_CREATE_INFO, pNext=None, flags=None, pApplicationInfo=None, enabledLayerCount=None, ppEnabledLayerNames=[], enabledExtensionCount=None, ppEnabledExtensionNames=[]):
	if enabledLayerCount is None:
		enabledLayerCount = len(ppEnabledLayerNames)
	if enabledExtensionCount is None:
		enabledExtensionCount = len(ppEnabledExtensionNames)
	return _new('VkInstanceCreateInfo', sType=sType, pNext=pNext, flags=flags, pApplicationInfo=pApplicationInfo, enabledLayerCount=enabledLayerCount, ppEnabledLayerNames=ppEnabledLayerNames, enabledExtensionCount=enabledExtensionCount, ppEnabledExtensionNames=ppEnabledExtensionNames)

def VkQueueFamilyProperties(queueFlags=None, queueCount=None, timestampValidBits=None, minImageTransferGranularity=None):
	return _new('VkQueueFamilyProperties', queueFlags=queueFlags, queueCount=queueCount, timestampValidBits=timestampValidBits, minImageTransferGranularity=minImageTransferGranularity)

def VkMemoryType(propertyFlags=None, heapIndex=None):
	return _new('VkMemoryType', propertyFlags=propertyFlags, heapIndex=heapIndex)

def VkMemoryHeap(size=None, flags=None):
	return _new('VkMemoryHeap', size=size, flags=flags)

def VkPhysicalDeviceMemoryProperties(memoryTypeCount=None, memoryTypes=None, memoryHeapCount=None, memoryHeaps=None):
	return _new('VkPhysicalDeviceMemoryProperties', memoryTypeCount=memoryTypeCount, memoryTypes=memoryTypes, memoryHeapCount=memoryHeapCount, memoryHeaps=memoryHeaps)

def VkMemoryAllocateInfo(sType=VK_STRUCTURE_TYPE_MEMORY_ALLOCATE_INFO, pNext=None, allocationSize=None, memoryTypeIndex=None):
	return _new('VkMemoryAllocateInfo', sType=sType, pNext=pNext, allocationSize=allocationSize, memoryTypeIndex=memoryTypeIndex)

def VkMemoryRequirements(size=None, alignment=None, memoryTypeBits=None):
	return _new('VkMemoryRequirements', size=size, alignment=alignment, memoryTypeBits=memoryTypeBits)

def VkSparseImageFormatProperties(aspectMask=None, imageGranularity=None, flags=None):
	return _new('VkSparseImageFormatProperties', aspectMask=aspectMask, imageGranularity=imageGranularity, flags=flags)

def VkSparseImageMemoryRequirements(formatProperties=None, imageMipTailFirstLod=None, imageMipTailSize=None, imageMipTailOffset=None, imageMipTailStride=None):
	return _new('VkSparseImageMemoryRequirements', formatProperties=formatProperties, imageMipTailFirstLod=imageMipTailFirstLod, imageMipTailSize=imageMipTailSize, imageMipTailOffset=imageMipTailOffset, imageMipTailStride=imageMipTailStride)

def VkMappedMemoryRange(sType=VK_STRUCTURE_TYPE_MAPPED_MEMORY_RANGE, pNext=None, memory=None, offset=None, size=None):
	return _new('VkMappedMemoryRange', sType=sType, pNext=pNext, memory=memory, offset=offset, size=size)

def VkFormatProperties(linearTilingFeatures=None, optimalTilingFeatures=None, bufferFeatures=None):
	return _new('VkFormatProperties', linearTilingFeatures=linearTilingFeatures, optimalTilingFeatures=optimalTilingFeatures, bufferFeatures=bufferFeatures)

def VkImageFormatProperties(maxExtent=None, maxMipLevels=None, maxArrayLayers=None, sampleCounts=None, maxResourceSize=None):
	return _new('VkImageFormatProperties', maxExtent=maxExtent, maxMipLevels=maxMipLevels, maxArrayLayers=maxArrayLayers, sampleCounts=sampleCounts, maxResourceSize=maxResourceSize)

def VkDescriptorBufferInfo(buffer=None, offset=None, range=None):
	return _new('VkDescriptorBufferInfo', buffer=buffer, offset=offset, range=range)

def VkDescriptorImageInfo(sampler=None, imageView=None, imageLayout=None):
	return _new('VkDescriptorImageInfo', sampler=sampler, imageView=imageView, imageLayout=imageLayout)

def VkWriteDescriptorSet(sType=VK_STRUCTURE_TYPE_WRITE_DESCRIPTOR_SET, pNext=None, dstSet=None, dstBinding=None, dstArrayElement=None, descriptorCount=None, descriptorType=None, pImageInfo=[], pBufferInfo=[], pTexelBufferView=[]):
	if descriptorCount is None:
		assert len(pImageInfo)==len(pBufferInfo)==len(pTexelBufferView)
		descriptorCount = len(pImageInfo)
	return _new('VkWriteDescriptorSet', sType=sType, pNext=pNext, dstSet=dstSet, dstBinding=dstBinding, dstArrayElement=dstArrayElement, descriptorCount=descriptorCount, descriptorType=descriptorType, pImageInfo=pImageInfo, pBufferInfo=pBufferInfo, pTexelBufferView=pTexelBufferView)

def VkCopyDescriptorSet(sType=VK_STRUCTURE_TYPE_COPY_DESCRIPTOR_SET, pNext=None, srcSet=None, srcBinding=None, srcArrayElement=None, dstSet=None, dstBinding=None, dstArrayElement=None, descriptorCount=None):
	return _new('VkCopyDescriptorSet', sType=sType, pNext=pNext, srcSet=srcSet, srcBinding=srcBinding, srcArrayElement=srcArrayElement, dstSet=dstSet, dstBinding=dstBinding, dstArrayElement=dstArrayElement, descriptorCount=descriptorCount)

def VkBufferCreateInfo(sType=VK_STRUCTURE_TYPE_BUFFER_CREATE_INFO, pNext=None, flags=None, size=None, usage=None, sharingMode=None, queueFamilyIndexCount=None, pQueueFamilyIndices=[]):
	if queueFamilyIndexCount is None:
		queueFamilyIndexCount = len(pQueueFamilyIndices)
	return _new('VkBufferCreateInfo', sType=sType, pNext=pNext, flags=flags, size=size, usage=usage, sharingMode=sharingMode, queueFamilyIndexCount=queueFamilyIndexCount, pQueueFamilyIndices=pQueueFamilyIndices)

def VkBufferViewCreateInfo(sType=VK_STRUCTURE_TYPE_BUFFER_VIEW_CREATE_INFO, pNext=None, flags=None, buffer=None, format=None, offset=None, range=None):
	return _new('VkBufferViewCreateInfo', sType=sType, pNext=pNext, flags=flags, buffer=buffer, format=format, offset=offset, range=range)

def VkImageSubresource(aspectMask=None, mipLevel=None, arrayLayer=None):
	return _new('VkImageSubresource', aspectMask=aspectMask, mipLevel=mipLevel, arrayLayer=arrayLayer)

def VkImageSubresourceLayers(aspectMask=None, mipLevel=None, baseArrayLayer=None, layerCount=None):
	return _new('VkImageSubresourceLayers', aspectMask=aspectMask, mipLevel=mipLevel, baseArrayLayer=baseArrayLayer, layerCount=layerCount)

def VkImageSubresourceRange(aspectMask=None, baseMipLevel=None, levelCount=None, baseArrayLayer=None, layerCount=None):
	return _new('VkImageSubresourceRange', aspectMask=aspectMask, baseMipLevel=baseMipLevel, levelCount=levelCount, baseArrayLayer=baseArrayLayer, layerCount=layerCount)

def VkMemoryBarrier(sType=VK_STRUCTURE_TYPE_MEMORY_BARRIER, pNext=None, srcAccessMask=None, dstAccessMask=None):
	return _new('VkMemoryBarrier', sType=sType, pNext=pNext, srcAccessMask=srcAccessMask, dstAccessMask=dstAccessMask)

def VkBufferMemoryBarrier(sType=VK_STRUCTURE_TYPE_BUFFER_MEMORY_BARRIER, pNext=None, srcAccessMask=None, dstAccessMask=None, srcQueueFamilyIndex=None, dstQueueFamilyIndex=None, buffer=None, offset=None, size=None):
	return _new('VkBufferMemoryBarrier', sType=sType, pNext=pNext, srcAccessMask=srcAccessMask, dstAccessMask=dstAccessMask, srcQueueFamilyIndex=srcQueueFamilyIndex, dstQueueFamilyIndex=dstQueueFamilyIndex, buffer=buffer, offset=offset, size=size)

def VkImageMemoryBarrier(sType=VK_STRUCTURE_TYPE_IMAGE_MEMORY_BARRIER, pNext=None, srcAccessMask=None, dstAccessMask=None, oldLayout=None, newLayout=None, srcQueueFamilyIndex=None, dstQueueFamilyIndex=None, image=None, subresourceRange=None):
	return _new('VkImageMemoryBarrier', sType=sType, pNext=pNext, srcAccessMask=srcAccessMask, dstAccessMask=dstAccessMask, oldLayout=oldLayout, newLayout=newLayout, srcQueueFamilyIndex=srcQueueFamilyIndex, dstQueueFamilyIndex=dstQueueFamilyIndex, image=image, subresourceRange=subresourceRange)

def VkImageCreateInfo(sType=VK_STRUCTURE_TYPE_IMAGE_CREATE_INFO, pNext=None, flags=None, imageType=None, format=None, extent=None, mipLevels=None, arrayLayers=None, samples=None, tiling=None, usage=None, sharingMode=None, queueFamilyIndexCount=None, pQueueFamilyIndices=[], initialLayout=None):
	if queueFamilyIndexCount is None:
		queueFamilyIndexCount = len(pQueueFamilyIndices)
	return _new('VkImageCreateInfo', sType=sType, pNext=pNext, flags=flags, imageType=imageType, format=format, extent=extent, mipLevels=mipLevels, arrayLayers=arrayLayers, samples=samples, tiling=tiling, usage=usage, sharingMode=sharingMode, queueFamilyIndexCount=queueFamilyIndexCount, pQueueFamilyIndices=pQueueFamilyIndices, initialLayout=initialLayout)

def VkSubresourceLayout(offset=None, size=None, rowPitch=None, arrayPitch=None, depthPitch=None):
	return _new('VkSubresourceLayout', offset=offset, size=size, rowPitch=rowPitch, arrayPitch=arrayPitch, depthPitch=depthPitch)

def VkImageViewCreateInfo(sType=VK_STRUCTURE_TYPE_IMAGE_VIEW_CREATE_INFO, pNext=None, flags=None, image=None, viewType=None, format=None, components=None, subresourceRange=None):
	return _new('VkImageViewCreateInfo', sType=sType, pNext=pNext, flags=flags, image=image, viewType=viewType, format=format, components=components, subresourceRange=subresourceRange)

def VkBufferCopy(srcOffset=None, dstOffset=None, size=None):
	return _new('VkBufferCopy', srcOffset=srcOffset, dstOffset=dstOffset, size=size)

def VkSparseMemoryBind(resourceOffset=None, size=None, memory=None, memoryOffset=None, flags=None):
	return _new('VkSparseMemoryBind', resourceOffset=resourceOffset, size=size, memory=memory, memoryOffset=memoryOffset, flags=flags)

def VkSparseImageMemoryBind(subresource=None, offset=None, extent=None, memory=None, memoryOffset=None, flags=None):
	return _new('VkSparseImageMemoryBind', subresource=subresource, offset=offset, extent=extent, memory=memory, memoryOffset=memoryOffset, flags=flags)

def VkSparseBufferMemoryBindInfo(buffer=None, bindCount=None, pBinds=[]):
	if bindCount is None:
		bindCount = len(pBinds)
	return _new('VkSparseBufferMemoryBindInfo', buffer=buffer, bindCount=bindCount, pBinds=pBinds)

def VkSparseImageOpaqueMemoryBindInfo(image=None, bindCount=None, pBinds=[]):
	if bindCount is None:
		bindCount = len(pBinds)
	return _new('VkSparseImageOpaqueMemoryBindInfo', image=image, bindCount=bindCount, pBinds=pBinds)

def VkSparseImageMemoryBindInfo(image=None, bindCount=None, pBinds=[]):
	if bindCount is None:
		bindCount = len(pBinds)
	return _new('VkSparseImageMemoryBindInfo', image=image, bindCount=bindCount, pBinds=pBinds)

def VkBindSparseInfo(sType=VK_STRUCTURE_TYPE_BIND_SPARSE_INFO, pNext=None, waitSemaphoreCount=None, pWaitSemaphores=[], bufferBindCount=None, pBufferBinds=[], imageOpaqueBindCount=None, pImageOpaqueBinds=[], imageBindCount=None, pImageBinds=[], signalSemaphoreCount=None, pSignalSemaphores=[]):
	if waitSemaphoreCount is None:
		waitSemaphoreCount = len(pWaitSemaphores)
	if bufferBindCount is None:
		bufferBindCount = len(pBufferBinds)
	if imageOpaqueBindCount is None:
		imageOpaqueBindCount = len(pImageOpaqueBinds)
	if imageBindCount is None:
		imageBindCount = len(pImageBinds)
	if signalSemaphoreCount is None:
		signalSemaphoreCount = len(pSignalSemaphores)
	return _new('VkBindSparseInfo', sType=sType, pNext=pNext, waitSemaphoreCount=waitSemaphoreCount, pWaitSemaphores=pWaitSemaphores, bufferBindCount=bufferBindCount, pBufferBinds=pBufferBinds, imageOpaqueBindCount=imageOpaqueBindCount, pImageOpaqueBinds=pImageOpaqueBinds, imageBindCount=imageBindCount, pImageBinds=pImageBinds, signalSemaphoreCount=signalSemaphoreCount, pSignalSemaphores=pSignalSemaphores)

def VkImageCopy(srcSubresource=None, srcOffset=None, dstSubresource=None, dstOffset=None, extent=None):
	return _new('VkImageCopy', srcSubresource=srcSubresource, srcOffset=srcOffset, dstSubresource=dstSubresource, dstOffset=dstOffset, extent=extent)

def VkImageBlit(srcSubresource=None, srcOffsets=None, dstSubresource=None, dstOffsets=None):
	return _new('VkImageBlit', srcSubresource=srcSubresource, srcOffsets=srcOffsets, dstSubresource=dstSubresource, dstOffsets=dstOffsets)

def VkBufferImageCopy(bufferOffset=None, bufferRowLength=None, bufferImageHeight=None, imageSubresource=None, imageOffset=None, imageExtent=None):
	return _new('VkBufferImageCopy', bufferOffset=bufferOffset, bufferRowLength=bufferRowLength, bufferImageHeight=bufferImageHeight, imageSubresource=imageSubresource, imageOffset=imageOffset, imageExtent=imageExtent)

def VkImageResolve(srcSubresource=None, srcOffset=None, dstSubresource=None, dstOffset=None, extent=None):
	return _new('VkImageResolve', srcSubresource=srcSubresource, srcOffset=srcOffset, dstSubresource=dstSubresource, dstOffset=dstOffset, extent=extent)

def VkShaderModuleCreateInfo(sType=VK_STRUCTURE_TYPE_SHADER_MODULE_CREATE_INFO, pNext=None, flags=None, codeSize=None, pCode=None):
	return _new('VkShaderModuleCreateInfo', sType=sType, pNext=pNext, flags=flags, codeSize=codeSize, pCode=pCode)

def VkDescriptorSetLayoutBinding(binding=None, descriptorType=None, descriptorCount=None, stageFlags=None, pImmutableSamplers=[]):
	if descriptorCount is None:
		descriptorCount = len(pImmutableSamplers)
	return _new('VkDescriptorSetLayoutBinding', binding=binding, descriptorType=descriptorType, descriptorCount=descriptorCount, stageFlags=stageFlags, pImmutableSamplers=pImmutableSamplers)

def VkDescriptorSetLayoutCreateInfo(sType=VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_CREATE_INFO, pNext=None, flags=None, bindingCount=None, pBindings=[]):
	if bindingCount is None:
		bindingCount = len(pBindings)
	return _new('VkDescriptorSetLayoutCreateInfo', sType=sType, pNext=pNext, flags=flags, bindingCount=bindingCount, pBindings=pBindings)

def VkDescriptorPoolSize(type=None, descriptorCount=None):
	return _new('VkDescriptorPoolSize', type=type, descriptorCount=descriptorCount)

def VkDescriptorPoolCreateInfo(sType=VK_STRUCTURE_TYPE_DESCRIPTOR_POOL_CREATE_INFO, pNext=None, flags=None, maxSets=None, poolSizeCount=None, pPoolSizes=[]):
	if poolSizeCount is None:
		poolSizeCount = len(pPoolSizes)
	return _new('VkDescriptorPoolCreateInfo', sType=sType, pNext=pNext, flags=flags, maxSets=maxSets, poolSizeCount=poolSizeCount, pPoolSizes=pPoolSizes)

def VkDescriptorSetAllocateInfo(sType=VK_STRUCTURE_TYPE_DESCRIPTOR_SET_ALLOCATE_INFO, pNext=None, descriptorPool=None, descriptorSetCount=None, pSetLayouts=[]):
	if descriptorSetCount is None:
		descriptorSetCount = len(pSetLayouts)
	return _new('VkDescriptorSetAllocateInfo', sType=sType, pNext=pNext, descriptorPool=descriptorPool, descriptorSetCount=descriptorSetCount, pSetLayouts=pSetLayouts)

def VkSpecializationMapEntry(constantID=None, offset=None, size=None):
	return _new('VkSpecializationMapEntry', constantID=constantID, offset=offset, size=size)

def VkSpecializationInfo(mapEntryCount=None, pMapEntries=[], dataSize=None, pData=[]):
	if mapEntryCount is None:
		mapEntryCount = len(pMapEntries)
	if dataSize is None:
		dataSize = len(pData)
	return _new('VkSpecializationInfo', mapEntryCount=mapEntryCount, pMapEntries=pMapEntries, dataSize=dataSize, pData=pData)

def VkPipelineShaderStageCreateInfo(sType=VK_STRUCTURE_TYPE_PIPELINE_SHADER_STAGE_CREATE_INFO, pNext=None, flags=None, stage=None, module=None, pName=None, pSpecializationInfo=None):
	return _new('VkPipelineShaderStageCreateInfo', sType=sType, pNext=pNext, flags=flags, stage=stage, module=module, pName=pName, pSpecializationInfo=pSpecializationInfo)

def VkComputePipelineCreateInfo(sType=VK_STRUCTURE_TYPE_COMPUTE_PIPELINE_CREATE_INFO, pNext=None, flags=None, stage=None, layout=None, basePipelineHandle=None, basePipelineIndex=None):
	return _new('VkComputePipelineCreateInfo', sType=sType, pNext=pNext, flags=flags, stage=stage, layout=layout, basePipelineHandle=basePipelineHandle, basePipelineIndex=basePipelineIndex)

def VkVertexInputBindingDescription(binding=None, stride=None, inputRate=None):
	return _new('VkVertexInputBindingDescription', binding=binding, stride=stride, inputRate=inputRate)

def VkVertexInputAttributeDescription(location=None, binding=None, format=None, offset=None):
	return _new('VkVertexInputAttributeDescription', location=location, binding=binding, format=format, offset=offset)

def VkPipelineVertexInputStateCreateInfo(sType=VK_STRUCTURE_TYPE_PIPELINE_VERTEX_INPUT_STATE_CREATE_INFO, pNext=None, flags=None, vertexBindingDescriptionCount=None, pVertexBindingDescriptions=[], vertexAttributeDescriptionCount=None, pVertexAttributeDescriptions=[]):
	if vertexBindingDescriptionCount is None:
		vertexBindingDescriptionCount = len(pVertexBindingDescriptions)
	if vertexAttributeDescriptionCount is None:
		vertexAttributeDescriptionCount = len(pVertexAttributeDescriptions)
	return _new('VkPipelineVertexInputStateCreateInfo', sType=sType, pNext=pNext, flags=flags, vertexBindingDescriptionCount=vertexBindingDescriptionCount, pVertexBindingDescriptions=pVertexBindingDescriptions, vertexAttributeDescriptionCount=vertexAttributeDescriptionCount, pVertexAttributeDescriptions=pVertexAttributeDescriptions)

def VkPipelineInputAssemblyStateCreateInfo(sType=VK_STRUCTURE_TYPE_PIPELINE_INPUT_ASSEMBLY_STATE_CREATE_INFO, pNext=None, flags=None, topology=None, primitiveRestartEnable=None):
	return _new('VkPipelineInputAssemblyStateCreateInfo', sType=sType, pNext=pNext, flags=flags, topology=topology, primitiveRestartEnable=primitiveRestartEnable)

def VkPipelineTessellationStateCreateInfo(sType=VK_STRUCTURE_TYPE_PIPELINE_TESSELLATION_STATE_CREATE_INFO, pNext=None, flags=None, patchControlPoints=None):
	return _new('VkPipelineTessellationStateCreateInfo', sType=sType, pNext=pNext, flags=flags, patchControlPoints=patchControlPoints)

def VkPipelineViewportStateCreateInfo(sType=VK_STRUCTURE_TYPE_PIPELINE_VIEWPORT_STATE_CREATE_INFO, pNext=None, flags=None, viewportCount=None, pViewports=[], scissorCount=None, pScissors=[]):
	if viewportCount is None:
		viewportCount = len(pViewports)
	if scissorCount is None:
		scissorCount = len(pScissors)
	return _new('VkPipelineViewportStateCreateInfo', sType=sType, pNext=pNext, flags=flags, viewportCount=viewportCount, pViewports=pViewports, scissorCount=scissorCount, pScissors=pScissors)

def VkPipelineRasterizationStateCreateInfo(sType=VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO, pNext=None, flags=None, depthClampEnable=None, rasterizerDiscardEnable=None, polygonMode=None, cullMode=None, frontFace=None, depthBiasEnable=None, depthBiasConstantFactor=None, depthBiasClamp=None, depthBiasSlopeFactor=None, lineWidth=None):
	return _new('VkPipelineRasterizationStateCreateInfo', sType=sType, pNext=pNext, flags=flags, depthClampEnable=depthClampEnable, rasterizerDiscardEnable=rasterizerDiscardEnable, polygonMode=polygonMode, cullMode=cullMode, frontFace=frontFace, depthBiasEnable=depthBiasEnable, depthBiasConstantFactor=depthBiasConstantFactor, depthBiasClamp=depthBiasClamp, depthBiasSlopeFactor=depthBiasSlopeFactor, lineWidth=lineWidth)

def VkPipelineMultisampleStateCreateInfo(sType=VK_STRUCTURE_TYPE_PIPELINE_MULTISAMPLE_STATE_CREATE_INFO, pNext=None, flags=None, rasterizationSamples=None, sampleShadingEnable=None, minSampleShading=None, pSampleMask=None, alphaToCoverageEnable=None, alphaToOneEnable=None):
	return _new('VkPipelineMultisampleStateCreateInfo', sType=sType, pNext=pNext, flags=flags, rasterizationSamples=rasterizationSamples, sampleShadingEnable=sampleShadingEnable, minSampleShading=minSampleShading, pSampleMask=pSampleMask, alphaToCoverageEnable=alphaToCoverageEnable, alphaToOneEnable=alphaToOneEnable)

def VkPipelineColorBlendAttachmentState(blendEnable=None, srcColorBlendFactor=None, dstColorBlendFactor=None, colorBlendOp=None, srcAlphaBlendFactor=None, dstAlphaBlendFactor=None, alphaBlendOp=None, colorWriteMask=None):
	return _new('VkPipelineColorBlendAttachmentState', blendEnable=blendEnable, srcColorBlendFactor=srcColorBlendFactor, dstColorBlendFactor=dstColorBlendFactor, colorBlendOp=colorBlendOp, srcAlphaBlendFactor=srcAlphaBlendFactor, dstAlphaBlendFactor=dstAlphaBlendFactor, alphaBlendOp=alphaBlendOp, colorWriteMask=colorWriteMask)

def VkPipelineColorBlendStateCreateInfo(sType=VK_STRUCTURE_TYPE_PIPELINE_COLOR_BLEND_STATE_CREATE_INFO, pNext=None, flags=None, logicOpEnable=None, logicOp=None, attachmentCount=None, pAttachments=[], blendConstants=None):
	if attachmentCount is None:
		attachmentCount = len(pAttachments)
	return _new('VkPipelineColorBlendStateCreateInfo', sType=sType, pNext=pNext, flags=flags, logicOpEnable=logicOpEnable, logicOp=logicOp, attachmentCount=attachmentCount, pAttachments=pAttachments, blendConstants=blendConstants)

def VkPipelineDynamicStateCreateInfo(sType=VK_STRUCTURE_TYPE_PIPELINE_DYNAMIC_STATE_CREATE_INFO, pNext=None, flags=None, dynamicStateCount=None, pDynamicStates=[]):
	if dynamicStateCount is None:
		dynamicStateCount = len(pDynamicStates)
	return _new('VkPipelineDynamicStateCreateInfo', sType=sType, pNext=pNext, flags=flags, dynamicStateCount=dynamicStateCount, pDynamicStates=pDynamicStates)

def VkStencilOpState(failOp=None, passOp=None, depthFailOp=None, compareOp=None, compareMask=None, writeMask=None, reference=None):
	return _new('VkStencilOpState', failOp=failOp, passOp=passOp, depthFailOp=depthFailOp, compareOp=compareOp, compareMask=compareMask, writeMask=writeMask, reference=reference)

def VkPipelineDepthStencilStateCreateInfo(sType=VK_STRUCTURE_TYPE_PIPELINE_DEPTH_STENCIL_STATE_CREATE_INFO, pNext=None, flags=None, depthTestEnable=None, depthWriteEnable=None, depthCompareOp=None, depthBoundsTestEnable=None, stencilTestEnable=None, front=None, back=None, minDepthBounds=None, maxDepthBounds=None):
	return _new('VkPipelineDepthStencilStateCreateInfo', sType=sType, pNext=pNext, flags=flags, depthTestEnable=depthTestEnable, depthWriteEnable=depthWriteEnable, depthCompareOp=depthCompareOp, depthBoundsTestEnable=depthBoundsTestEnable, stencilTestEnable=stencilTestEnable, front=front, back=back, minDepthBounds=minDepthBounds, maxDepthBounds=maxDepthBounds)

def VkGraphicsPipelineCreateInfo(sType=VK_STRUCTURE_TYPE_GRAPHICS_PIPELINE_CREATE_INFO, pNext=None, flags=None, stageCount=None, pStages=[], pVertexInputState=None, pInputAssemblyState=None, pTessellationState=None, pViewportState=None, pRasterizationState=None, pMultisampleState=None, pDepthStencilState=None, pColorBlendState=None, pDynamicState=None, layout=None, renderPass=None, subpass=None, basePipelineHandle=None, basePipelineIndex=None):
	if stageCount is None:
		stageCount = len(pStages)
	return _new('VkGraphicsPipelineCreateInfo', sType=sType, pNext=pNext, flags=flags, stageCount=stageCount, pStages=pStages, pVertexInputState=pVertexInputState, pInputAssemblyState=pInputAssemblyState, pTessellationState=pTessellationState, pViewportState=pViewportState, pRasterizationState=pRasterizationState, pMultisampleState=pMultisampleState, pDepthStencilState=pDepthStencilState, pColorBlendState=pColorBlendState, pDynamicState=pDynamicState, layout=layout, renderPass=renderPass, subpass=subpass, basePipelineHandle=basePipelineHandle, basePipelineIndex=basePipelineIndex)

def VkPipelineCacheCreateInfo(sType=VK_STRUCTURE_TYPE_PIPELINE_CACHE_CREATE_INFO, pNext=None, flags=None, initialDataSize=None, pInitialData=[]):
	if initialDataSize is None:
		initialDataSize = len(pInitialData)
	return _new('VkPipelineCacheCreateInfo', sType=sType, pNext=pNext, flags=flags, initialDataSize=initialDataSize, pInitialData=pInitialData)

def VkPushConstantRange(stageFlags=None, offset=None, size=None):
	return _new('VkPushConstantRange', stageFlags=stageFlags, offset=offset, size=size)

def VkPipelineLayoutCreateInfo(sType=VK_STRUCTURE_TYPE_PIPELINE_LAYOUT_CREATE_INFO, pNext=None, flags=None, setLayoutCount=None, pSetLayouts=[], pushConstantRangeCount=None, pPushConstantRanges=[]):
	if setLayoutCount is None:
		setLayoutCount = len(pSetLayouts)
	if pushConstantRangeCount is None:
		pushConstantRangeCount = len(pPushConstantRanges)
	return _new('VkPipelineLayoutCreateInfo', sType=sType, pNext=pNext, flags=flags, setLayoutCount=setLayoutCount, pSetLayouts=pSetLayouts, pushConstantRangeCount=pushConstantRangeCount, pPushConstantRanges=pPushConstantRanges)

def VkSamplerCreateInfo(sType=VK_STRUCTURE_TYPE_SAMPLER_CREATE_INFO, pNext=None, flags=None, magFilter=None, minFilter=None, mipmapMode=None, addressModeU=None, addressModeV=None, addressModeW=None, mipLodBias=None, anisotropyEnable=None, maxAnisotropy=None, compareEnable=None, compareOp=None, minLod=None, maxLod=None, borderColor=None, unnormalizedCoordinates=None):
	return _new('VkSamplerCreateInfo', sType=sType, pNext=pNext, flags=flags, magFilter=magFilter, minFilter=minFilter, mipmapMode=mipmapMode, addressModeU=addressModeU, addressModeV=addressModeV, addressModeW=addressModeW, mipLodBias=mipLodBias, anisotropyEnable=anisotropyEnable, maxAnisotropy=maxAnisotropy, compareEnable=compareEnable, compareOp=compareOp, minLod=minLod, maxLod=maxLod, borderColor=borderColor, unnormalizedCoordinates=unnormalizedCoordinates)

def VkCommandPoolCreateInfo(sType=VK_STRUCTURE_TYPE_COMMAND_POOL_CREATE_INFO, pNext=None, flags=None, queueFamilyIndex=None):
	return _new('VkCommandPoolCreateInfo', sType=sType, pNext=pNext, flags=flags, queueFamilyIndex=queueFamilyIndex)

def VkCommandBufferAllocateInfo(sType=VK_STRUCTURE_TYPE_COMMAND_BUFFER_ALLOCATE_INFO, pNext=None, commandPool=None, level=None, commandBufferCount=None):
	return _new('VkCommandBufferAllocateInfo', sType=sType, pNext=pNext, commandPool=commandPool, level=level, commandBufferCount=commandBufferCount)

def VkCommandBufferInheritanceInfo(sType=VK_STRUCTURE_TYPE_COMMAND_BUFFER_INHERITANCE_INFO, pNext=None, renderPass=None, subpass=None, framebuffer=None, occlusionQueryEnable=None, queryFlags=None, pipelineStatistics=None):
	return _new('VkCommandBufferInheritanceInfo', sType=sType, pNext=pNext, renderPass=renderPass, subpass=subpass, framebuffer=framebuffer, occlusionQueryEnable=occlusionQueryEnable, queryFlags=queryFlags, pipelineStatistics=pipelineStatistics)

def VkCommandBufferBeginInfo(sType=VK_STRUCTURE_TYPE_COMMAND_BUFFER_BEGIN_INFO, pNext=None, flags=None, pInheritanceInfo=None):
	return _new('VkCommandBufferBeginInfo', sType=sType, pNext=pNext, flags=flags, pInheritanceInfo=pInheritanceInfo)

def VkClearColorValue(float32=None, int32=None, uint32=None):
	return _new('VkClearColorValue', float32=float32, int32=int32, uint32=uint32)

def VkClearDepthStencilValue(depth=None, stencil=None):
	return _new('VkClearDepthStencilValue', depth=depth, stencil=stencil)

def VkClearValue(color=None, depthStencil=None):
	return _new('VkClearValue', color=color, depthStencil=depthStencil)

def VkRenderPassBeginInfo(sType=VK_STRUCTURE_TYPE_RENDER_PASS_BEGIN_INFO, pNext=None, renderPass=None, framebuffer=None, renderArea=None, clearValueCount=None, pClearValues=[]):
	if clearValueCount is None:
		clearValueCount = len(pClearValues)
	return _new('VkRenderPassBeginInfo', sType=sType, pNext=pNext, renderPass=renderPass, framebuffer=framebuffer, renderArea=renderArea, clearValueCount=clearValueCount, pClearValues=pClearValues)

def VkClearAttachment(aspectMask=None, colorAttachment=None, clearValue=None):
	return _new('VkClearAttachment', aspectMask=aspectMask, colorAttachment=colorAttachment, clearValue=clearValue)

def VkAttachmentDescription(flags=None, format=None, samples=None, loadOp=None, storeOp=None, stencilLoadOp=None, stencilStoreOp=None, initialLayout=None, finalLayout=None):
	return _new('VkAttachmentDescription', flags=flags, format=format, samples=samples, loadOp=loadOp, storeOp=storeOp, stencilLoadOp=stencilLoadOp, stencilStoreOp=stencilStoreOp, initialLayout=initialLayout, finalLayout=finalLayout)

def VkAttachmentReference(attachment=None, layout=None):
	return _new('VkAttachmentReference', attachment=attachment, layout=layout)

def VkSubpassDescription(flags=None, pipelineBindPoint=None, inputAttachmentCount=None, pInputAttachments=[], colorAttachmentCount=None, pColorAttachments=[], pResolveAttachments=[], pDepthStencilAttachment=None, preserveAttachmentCount=None, pPreserveAttachments=[]):
	if inputAttachmentCount is None:
		inputAttachmentCount = len(pInputAttachments)
	if colorAttachmentCount is None:
		assert len(pColorAttachments)==len(pResolveAttachments)
		colorAttachmentCount = len(pColorAttachments)
	if preserveAttachmentCount is None:
		preserveAttachmentCount = len(pPreserveAttachments)
	return _new('VkSubpassDescription', flags=flags, pipelineBindPoint=pipelineBindPoint, inputAttachmentCount=inputAttachmentCount, pInputAttachments=pInputAttachments, colorAttachmentCount=colorAttachmentCount, pColorAttachments=pColorAttachments, pResolveAttachments=pResolveAttachments, pDepthStencilAttachment=pDepthStencilAttachment, preserveAttachmentCount=preserveAttachmentCount, pPreserveAttachments=pPreserveAttachments)

def VkSubpassDependency(srcSubpass=None, dstSubpass=None, srcStageMask=None, dstStageMask=None, srcAccessMask=None, dstAccessMask=None, dependencyFlags=None):
	return _new('VkSubpassDependency', srcSubpass=srcSubpass, dstSubpass=dstSubpass, srcStageMask=srcStageMask, dstStageMask=dstStageMask, srcAccessMask=srcAccessMask, dstAccessMask=dstAccessMask, dependencyFlags=dependencyFlags)

def VkRenderPassCreateInfo(sType=VK_STRUCTURE_TYPE_RENDER_PASS_CREATE_INFO, pNext=None, flags=None, attachmentCount=None, pAttachments=[], subpassCount=None, pSubpasses=[], dependencyCount=None, pDependencies=[]):
	if attachmentCount is None:
		attachmentCount = len(pAttachments)
	if subpassCount is None:
		subpassCount = len(pSubpasses)
	if dependencyCount is None:
		dependencyCount = len(pDependencies)
	return _new('VkRenderPassCreateInfo', sType=sType, pNext=pNext, flags=flags, attachmentCount=attachmentCount, pAttachments=pAttachments, subpassCount=subpassCount, pSubpasses=pSubpasses, dependencyCount=dependencyCount, pDependencies=pDependencies)

def VkEventCreateInfo(sType=VK_STRUCTURE_TYPE_EVENT_CREATE_INFO, pNext=None, flags=None):
	return _new('VkEventCreateInfo', sType=sType, pNext=pNext, flags=flags)

def VkFenceCreateInfo(sType=VK_STRUCTURE_TYPE_FENCE_CREATE_INFO, pNext=None, flags=None):
	return _new('VkFenceCreateInfo', sType=sType, pNext=pNext, flags=flags)

def VkSemaphoreCreateInfo(sType=VK_STRUCTURE_TYPE_SEMAPHORE_CREATE_INFO, pNext=None, flags=None):
	return _new('VkSemaphoreCreateInfo', sType=sType, pNext=pNext, flags=flags)

def VkQueryPoolCreateInfo(sType=VK_STRUCTURE_TYPE_QUERY_POOL_CREATE_INFO, pNext=None, flags=None, queryType=None, queryCount=None, pipelineStatistics=None):
	return _new('VkQueryPoolCreateInfo', sType=sType, pNext=pNext, flags=flags, queryType=queryType, queryCount=queryCount, pipelineStatistics=pipelineStatistics)

def VkFramebufferCreateInfo(sType=VK_STRUCTURE_TYPE_FRAMEBUFFER_CREATE_INFO, pNext=None, flags=None, renderPass=None, attachmentCount=None, pAttachments=[], width=None, height=None, layers=None):
	if attachmentCount is None:
		attachmentCount = len(pAttachments)
	return _new('VkFramebufferCreateInfo', sType=sType, pNext=pNext, flags=flags, renderPass=renderPass, attachmentCount=attachmentCount, pAttachments=pAttachments, width=width, height=height, layers=layers)

def VkDrawIndirectCommand(vertexCount=None, instanceCount=None, firstVertex=None, firstInstance=None):
	return _new('VkDrawIndirectCommand', vertexCount=vertexCount, instanceCount=instanceCount, firstVertex=firstVertex, firstInstance=firstInstance)

def VkDrawIndexedIndirectCommand(indexCount=None, instanceCount=None, firstIndex=None, vertexOffset=None, firstInstance=None):
	return _new('VkDrawIndexedIndirectCommand', indexCount=indexCount, instanceCount=instanceCount, firstIndex=firstIndex, vertexOffset=vertexOffset, firstInstance=firstInstance)

def VkDispatchIndirectCommand(x=None, y=None, z=None):
	return _new('VkDispatchIndirectCommand', x=x, y=y, z=z)

def VkSubmitInfo(sType=VK_STRUCTURE_TYPE_SUBMIT_INFO, pNext=None, waitSemaphoreCount=None, pWaitSemaphores=[], pWaitDstStageMask=[], commandBufferCount=None, pCommandBuffers=[], signalSemaphoreCount=None, pSignalSemaphores=[]):
	if waitSemaphoreCount is None:
		assert len(pWaitSemaphores)==len(pWaitDstStageMask)
		waitSemaphoreCount = len(pWaitSemaphores)
	if commandBufferCount is None:
		commandBufferCount = len(pCommandBuffers)
	if signalSemaphoreCount is None:
		signalSemaphoreCount = len(pSignalSemaphores)
	return _new('VkSubmitInfo', sType=sType, pNext=pNext, waitSemaphoreCount=waitSemaphoreCount, pWaitSemaphores=pWaitSemaphores, pWaitDstStageMask=pWaitDstStageMask, commandBufferCount=commandBufferCount, pCommandBuffers=pCommandBuffers, signalSemaphoreCount=signalSemaphoreCount, pSignalSemaphores=pSignalSemaphores)

def VkDisplayPropertiesKHR(display=None, displayName=None, physicalDimensions=None, physicalResolution=None, supportedTransforms=None, planeReorderPossible=None, persistentContent=None):
	return _new('VkDisplayPropertiesKHR', display=display, displayName=displayName, physicalDimensions=physicalDimensions, physicalResolution=physicalResolution, supportedTransforms=supportedTransforms, planeReorderPossible=planeReorderPossible, persistentContent=persistentContent)

def VkDisplayPlanePropertiesKHR(currentDisplay=None, currentStackIndex=None):
	return _new('VkDisplayPlanePropertiesKHR', currentDisplay=currentDisplay, currentStackIndex=currentStackIndex)

def VkDisplayModeParametersKHR(visibleRegion=None, refreshRate=None):
	return _new('VkDisplayModeParametersKHR', visibleRegion=visibleRegion, refreshRate=refreshRate)

def VkDisplayModePropertiesKHR(displayMode=None, parameters=None):
	return _new('VkDisplayModePropertiesKHR', displayMode=displayMode, parameters=parameters)

def VkDisplayModeCreateInfoKHR(sType=VK_STRUCTURE_TYPE_DISPLAY_MODE_CREATE_INFO_KHR, pNext=None, flags=None, parameters=None):
	return _new('VkDisplayModeCreateInfoKHR', sType=sType, pNext=pNext, flags=flags, parameters=parameters)

def VkDisplayPlaneCapabilitiesKHR(supportedAlpha=None, minSrcPosition=None, maxSrcPosition=None, minSrcExtent=None, maxSrcExtent=None, minDstPosition=None, maxDstPosition=None, minDstExtent=None, maxDstExtent=None):
	return _new('VkDisplayPlaneCapabilitiesKHR', supportedAlpha=supportedAlpha, minSrcPosition=minSrcPosition, maxSrcPosition=maxSrcPosition, minSrcExtent=minSrcExtent, maxSrcExtent=maxSrcExtent, minDstPosition=minDstPosition, maxDstPosition=maxDstPosition, minDstExtent=minDstExtent, maxDstExtent=maxDstExtent)

def VkDisplaySurfaceCreateInfoKHR(sType=VK_STRUCTURE_TYPE_DISPLAY_SURFACE_CREATE_INFO_KHR, pNext=None, flags=None, displayMode=None, planeIndex=None, planeStackIndex=None, transform=None, globalAlpha=None, alphaMode=None, imageExtent=None):
	return _new('VkDisplaySurfaceCreateInfoKHR', sType=sType, pNext=pNext, flags=flags, displayMode=displayMode, planeIndex=planeIndex, planeStackIndex=planeStackIndex, transform=transform, globalAlpha=globalAlpha, alphaMode=alphaMode, imageExtent=imageExtent)

def VkDisplayPresentInfoKHR(sType=VK_STRUCTURE_TYPE_DISPLAY_PRESENT_INFO_KHR, pNext=None, srcRect=None, dstRect=None, persistent=None):
	return _new('VkDisplayPresentInfoKHR', sType=sType, pNext=pNext, srcRect=srcRect, dstRect=dstRect, persistent=persistent)

def VkSurfaceCapabilitiesKHR(minImageCount=None, maxImageCount=None, currentExtent=None, minImageExtent=None, maxImageExtent=None, maxImageArrayLayers=None, supportedTransforms=None, currentTransform=None, supportedCompositeAlpha=None, supportedUsageFlags=None):
	return _new('VkSurfaceCapabilitiesKHR', minImageCount=minImageCount, maxImageCount=maxImageCount, currentExtent=currentExtent, minImageExtent=minImageExtent, maxImageExtent=maxImageExtent, maxImageArrayLayers=maxImageArrayLayers, supportedTransforms=supportedTransforms, currentTransform=currentTransform, supportedCompositeAlpha=supportedCompositeAlpha, supportedUsageFlags=supportedUsageFlags)

def VkAndroidSurfaceCreateInfoKHR(sType=VK_STRUCTURE_TYPE_ANDROID_SURFACE_CREATE_INFO_KHR, pNext=None, flags=None, window=None):
	return _new('VkAndroidSurfaceCreateInfoKHR', sType=sType, pNext=pNext, flags=flags, window=window)

def VkMirSurfaceCreateInfoKHR(sType=VK_STRUCTURE_TYPE_MIR_SURFACE_CREATE_INFO_KHR, pNext=None, flags=None, connection=None, mirSurface=None):
	return _new('VkMirSurfaceCreateInfoKHR', sType=sType, pNext=pNext, flags=flags, connection=connection, mirSurface=mirSurface)

def VkWaylandSurfaceCreateInfoKHR(sType=VK_STRUCTURE_TYPE_WAYLAND_SURFACE_CREATE_INFO_KHR, pNext=None, flags=None, display=None, surface=None):
	return _new('VkWaylandSurfaceCreateInfoKHR', sType=sType, pNext=pNext, flags=flags, display=display, surface=surface)

def VkWin32SurfaceCreateInfoKHR(sType=VK_STRUCTURE_TYPE_WIN32_SURFACE_CREATE_INFO_KHR, pNext=None, flags=None, hinstance=None, hwnd=None):
	return _new('VkWin32SurfaceCreateInfoKHR', sType=sType, pNext=pNext, flags=flags, hinstance=hinstance, hwnd=hwnd)

def VkXlibSurfaceCreateInfoKHR(sType=VK_STRUCTURE_TYPE_XLIB_SURFACE_CREATE_INFO_KHR, pNext=None, flags=None, dpy=None, window=None):
	return _new('VkXlibSurfaceCreateInfoKHR', sType=sType, pNext=pNext, flags=flags, dpy=dpy, window=window)

def VkXcbSurfaceCreateInfoKHR(sType=VK_STRUCTURE_TYPE_XCB_SURFACE_CREATE_INFO_KHR, pNext=None, flags=None, connection=None, window=None):
	return _new('VkXcbSurfaceCreateInfoKHR', sType=sType, pNext=pNext, flags=flags, connection=connection, window=window)

def VkSurfaceFormatKHR(format=None, colorSpace=None):
	return _new('VkSurfaceFormatKHR', format=format, colorSpace=colorSpace)

def VkSwapchainCreateInfoKHR(sType=VK_STRUCTURE_TYPE_SWAPCHAIN_CREATE_INFO_KHR, pNext=None, flags=None, surface=None, minImageCount=None, imageFormat=None, imageColorSpace=None, imageExtent=None, imageArrayLayers=None, imageUsage=None, imageSharingMode=None, queueFamilyIndexCount=None, pQueueFamilyIndices=[], preTransform=None, compositeAlpha=None, presentMode=None, clipped=None, oldSwapchain=None):
	if queueFamilyIndexCount is None:
		queueFamilyIndexCount = len(pQueueFamilyIndices)
	return _new('VkSwapchainCreateInfoKHR', sType=sType, pNext=pNext, flags=flags, surface=surface, minImageCount=minImageCount, imageFormat=imageFormat, imageColorSpace=imageColorSpace, imageExtent=imageExtent, imageArrayLayers=imageArrayLayers, imageUsage=imageUsage, imageSharingMode=imageSharingMode, queueFamilyIndexCount=queueFamilyIndexCount, pQueueFamilyIndices=pQueueFamilyIndices, preTransform=preTransform, compositeAlpha=compositeAlpha, presentMode=presentMode, clipped=clipped, oldSwapchain=oldSwapchain)

def VkPresentInfoKHR(sType=VK_STRUCTURE_TYPE_PRESENT_INFO_KHR, pNext=None, waitSemaphoreCount=None, pWaitSemaphores=[], swapchainCount=None, pSwapchains=[], pImageIndices=[], pResults=[]):
	if waitSemaphoreCount is None:
		waitSemaphoreCount = len(pWaitSemaphores)
	if swapchainCount is None:
		assert len(pSwapchains)==len(pImageIndices)==len(pResults)
		swapchainCount = len(pSwapchains)
	return _new('VkPresentInfoKHR', sType=sType, pNext=pNext, waitSemaphoreCount=waitSemaphoreCount, pWaitSemaphores=pWaitSemaphores, swapchainCount=swapchainCount, pSwapchains=pSwapchains, pImageIndices=pImageIndices, pResults=pResults)

def VkDebugReportCallbackCreateInfoEXT(sType=VK_STRUCTURE_TYPE_DEBUG_REPORT_CALLBACK_CREATE_INFO_EXT, pNext=None, flags=None, pfnCallback=None, pUserData=None):
	return _new('VkDebugReportCallbackCreateInfoEXT', sType=sType, pNext=pNext, flags=flags, pfnCallback=pfnCallback, pUserData=pUserData)

def VkValidationFlagsEXT(sType=None, pNext=None, disabledValidationCheckCount=None, pDisabledValidationChecks=[]):
	if disabledValidationCheckCount is None:
		disabledValidationCheckCount = len(pDisabledValidationChecks)
	return _new('VkValidationFlagsEXT', sType=sType, pNext=pNext, disabledValidationCheckCount=disabledValidationCheckCount, pDisabledValidationChecks=pDisabledValidationChecks)

def VkPipelineRasterizationStateRasterizationOrderAMD(sType=VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_RASTERIZATION_ORDER_AMD, pNext=None, rasterizationOrder=None):
	return _new('VkPipelineRasterizationStateRasterizationOrderAMD', sType=sType, pNext=pNext, rasterizationOrder=rasterizationOrder)

def VkDebugMarkerObjectNameInfoEXT(sType=VK_STRUCTURE_TYPE_DEBUG_MARKER_OBJECT_NAME_INFO_EXT, pNext=None, objectType=None, object=None, pObjectName=None):
	return _new('VkDebugMarkerObjectNameInfoEXT', sType=sType, pNext=pNext, objectType=objectType, object=object, pObjectName=pObjectName)

def VkDebugMarkerObjectTagInfoEXT(sType=VK_STRUCTURE_TYPE_DEBUG_MARKER_OBJECT_TAG_INFO_EXT, pNext=None, objectType=None, object=None, tagName=None, tagSize=None, pTag=[]):
	if tagSize is None:
		tagSize = len(pTag)
	return _new('VkDebugMarkerObjectTagInfoEXT', sType=sType, pNext=pNext, objectType=objectType, object=object, tagName=tagName, tagSize=tagSize, pTag=pTag)

def VkDebugMarkerMarkerInfoEXT(sType=VK_STRUCTURE_TYPE_DEBUG_MARKER_MARKER_INFO_EXT, pNext=None, pMarkerName=None, color=None):
	return _new('VkDebugMarkerMarkerInfoEXT', sType=sType, pNext=pNext, pMarkerName=pMarkerName, color=color)

def VkDedicatedAllocationImageCreateInfoNV(sType=VK_STRUCTURE_TYPE_DEDICATED_ALLOCATION_IMAGE_CREATE_INFO_NV, pNext=None, dedicatedAllocation=None):
	return _new('VkDedicatedAllocationImageCreateInfoNV', sType=sType, pNext=pNext, dedicatedAllocation=dedicatedAllocation)

def VkDedicatedAllocationBufferCreateInfoNV(sType=VK_STRUCTURE_TYPE_DEDICATED_ALLOCATION_BUFFER_CREATE_INFO_NV, pNext=None, dedicatedAllocation=None):
	return _new('VkDedicatedAllocationBufferCreateInfoNV', sType=sType, pNext=pNext, dedicatedAllocation=dedicatedAllocation)

def VkDedicatedAllocationMemoryAllocateInfoNV(sType=VK_STRUCTURE_TYPE_DEDICATED_ALLOCATION_MEMORY_ALLOCATE_INFO_NV, pNext=None, image=None, buffer=None):
	return _new('VkDedicatedAllocationMemoryAllocateInfoNV', sType=sType, pNext=pNext, image=image, buffer=buffer)

def VkExternalImageFormatPropertiesNV(imageFormatProperties=None, externalMemoryFeatures=None, exportFromImportedHandleTypes=None, compatibleHandleTypes=None):
	return _new('VkExternalImageFormatPropertiesNV', imageFormatProperties=imageFormatProperties, externalMemoryFeatures=externalMemoryFeatures, exportFromImportedHandleTypes=exportFromImportedHandleTypes, compatibleHandleTypes=compatibleHandleTypes)

def VkExternalMemoryImageCreateInfoNV(sType=VK_STRUCTURE_TYPE_EXTERNAL_MEMORY_IMAGE_CREATE_INFO_NV, pNext=None, handleTypes=None):
	return _new('VkExternalMemoryImageCreateInfoNV', sType=sType, pNext=pNext, handleTypes=handleTypes)

def VkExportMemoryAllocateInfoNV(sType=VK_STRUCTURE_TYPE_EXPORT_MEMORY_ALLOCATE_INFO_NV, pNext=None, handleTypes=None):
	return _new('VkExportMemoryAllocateInfoNV', sType=sType, pNext=pNext, handleTypes=handleTypes)

def VkImportMemoryWin32HandleInfoNV(sType=VK_STRUCTURE_TYPE_IMPORT_MEMORY_WIN32_HANDLE_INFO_NV, pNext=None, handleType=None, handle=None):
	return _new('VkImportMemoryWin32HandleInfoNV', sType=sType, pNext=pNext, handleType=handleType, handle=handle)

def VkExportMemoryWin32HandleInfoNV(sType=VK_STRUCTURE_TYPE_EXPORT_MEMORY_WIN32_HANDLE_INFO_NV, pNext=None, pAttributes=None, dwAccess=None):
	return _new('VkExportMemoryWin32HandleInfoNV', sType=sType, pNext=pNext, pAttributes=pAttributes, dwAccess=dwAccess)

def VkWin32KeyedMutexAcquireReleaseInfoNV(sType=VK_STRUCTURE_TYPE_WIN32_KEYED_MUTEX_ACQUIRE_RELEASE_INFO_NV, pNext=None, acquireCount=None, pAcquireSyncs=[], pAcquireKeys=[], pAcquireTimeoutMilliseconds=[], releaseCount=None, pReleaseSyncs=[], pReleaseKeys=[]):
	if acquireCount is None:
		assert len(pAcquireSyncs)==len(pAcquireKeys)==len(pAcquireTimeoutMilliseconds)
		acquireCount = len(pAcquireSyncs)
	if releaseCount is None:
		assert len(pReleaseSyncs)==len(pReleaseKeys)
		releaseCount = len(pReleaseSyncs)
	return _new('VkWin32KeyedMutexAcquireReleaseInfoNV', sType=sType, pNext=pNext, acquireCount=acquireCount, pAcquireSyncs=pAcquireSyncs, pAcquireKeys=pAcquireKeys, pAcquireTimeoutMilliseconds=pAcquireTimeoutMilliseconds, releaseCount=releaseCount, pReleaseSyncs=pReleaseSyncs, pReleaseKeys=pReleaseKeys)

def _callApi(fn, *args):
	def _(x, _type):
		if x is None:
			return ffi.NULL
		if _type.kind=='pointer':
			ptr, _ = _castToPtr(x, _type)
			return ptr
		return x

	return fn(*(_(i, j) for i, j in zip(args, ffi.typeof(fn).args)))






def vkCreateInstance(pCreateInfo, pAllocator):
	pInstance = ffi.new('VkInstance*')
	result = _callApi(_lib.vkCreateInstance, pCreateInfo, pAllocator, pInstance)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pInstance[0]

def vkGetPhysicalDeviceProperties(physicalDevice):
	pProperties = ffi.new('VkPhysicalDeviceProperties*')
	_callApi(_lib.vkGetPhysicalDeviceProperties, physicalDevice, pProperties)
	return pProperties[0]

def vkGetPhysicalDeviceMemoryProperties(physicalDevice):
	pMemoryProperties = ffi.new('VkPhysicalDeviceMemoryProperties*')
	_callApi(_lib.vkGetPhysicalDeviceMemoryProperties, physicalDevice, pMemoryProperties)
	return pMemoryProperties[0]

def vkGetPhysicalDeviceFeatures(physicalDevice):
	pFeatures = ffi.new('VkPhysicalDeviceFeatures*')
	_callApi(_lib.vkGetPhysicalDeviceFeatures, physicalDevice, pFeatures)
	return pFeatures[0]

def vkGetPhysicalDeviceFormatProperties(physicalDevice, format):
	pFormatProperties = ffi.new('VkFormatProperties*')
	_callApi(_lib.vkGetPhysicalDeviceFormatProperties, physicalDevice, format, pFormatProperties)
	return pFormatProperties[0]

def vkGetPhysicalDeviceImageFormatProperties(physicalDevice, format, type, tiling, usage, flags):
	pImageFormatProperties = ffi.new('VkImageFormatProperties*')
	result = _callApi(_lib.vkGetPhysicalDeviceImageFormatProperties, physicalDevice, format, type, tiling, usage, flags, pImageFormatProperties)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pImageFormatProperties[0]

def vkCreateDevice(physicalDevice, pCreateInfo, pAllocator):
	pDevice = ffi.new('VkDevice*')
	result = _callApi(_lib.vkCreateDevice, physicalDevice, pCreateInfo, pAllocator, pDevice)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pDevice[0]

def vkGetDeviceQueue(device, queueFamilyIndex, queueIndex):
	pQueue = ffi.new('VkQueue*')
	_callApi(_lib.vkGetDeviceQueue, device, queueFamilyIndex, queueIndex, pQueue)
	return pQueue[0]

def vkAllocateMemory(device, pAllocateInfo, pAllocator):
	pMemory = ffi.new('VkDeviceMemory*')
	result = _callApi(_lib.vkAllocateMemory, device, pAllocateInfo, pAllocator, pMemory)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pMemory[0]

def vkMapMemory(device, memory, offset, size, flags):
	ppData = ffi.new('void**')
	result = _callApi(_lib.vkMapMemory, device, memory, offset, size, flags, ppData)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return ppData[0]

def vkGetDeviceMemoryCommitment(device, memory):
	pCommittedMemoryInBytes = ffi.new('VkDeviceSize*')
	_callApi(_lib.vkGetDeviceMemoryCommitment, device, memory, pCommittedMemoryInBytes)
	return pCommittedMemoryInBytes[0]

def vkGetBufferMemoryRequirements(device, buffer):
	pMemoryRequirements = ffi.new('VkMemoryRequirements*')
	_callApi(_lib.vkGetBufferMemoryRequirements, device, buffer, pMemoryRequirements)
	return pMemoryRequirements[0]

def vkGetImageMemoryRequirements(device, image):
	pMemoryRequirements = ffi.new('VkMemoryRequirements*')
	_callApi(_lib.vkGetImageMemoryRequirements, device, image, pMemoryRequirements)
	return pMemoryRequirements[0]

def vkCreateFence(device, pCreateInfo, pAllocator):
	pFence = ffi.new('VkFence*')
	result = _callApi(_lib.vkCreateFence, device, pCreateInfo, pAllocator, pFence)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pFence[0]

def vkCreateSemaphore(device, pCreateInfo, pAllocator):
	pSemaphore = ffi.new('VkSemaphore*')
	result = _callApi(_lib.vkCreateSemaphore, device, pCreateInfo, pAllocator, pSemaphore)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pSemaphore[0]

def vkCreateEvent(device, pCreateInfo, pAllocator):
	pEvent = ffi.new('VkEvent*')
	result = _callApi(_lib.vkCreateEvent, device, pCreateInfo, pAllocator, pEvent)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pEvent[0]

def vkCreateQueryPool(device, pCreateInfo, pAllocator):
	pQueryPool = ffi.new('VkQueryPool*')
	result = _callApi(_lib.vkCreateQueryPool, device, pCreateInfo, pAllocator, pQueryPool)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pQueryPool[0]

def vkCreateBuffer(device, pCreateInfo, pAllocator):
	pBuffer = ffi.new('VkBuffer*')
	result = _callApi(_lib.vkCreateBuffer, device, pCreateInfo, pAllocator, pBuffer)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pBuffer[0]

def vkCreateBufferView(device, pCreateInfo, pAllocator):
	pView = ffi.new('VkBufferView*')
	result = _callApi(_lib.vkCreateBufferView, device, pCreateInfo, pAllocator, pView)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pView[0]

def vkCreateImage(device, pCreateInfo, pAllocator):
	pImage = ffi.new('VkImage*')
	result = _callApi(_lib.vkCreateImage, device, pCreateInfo, pAllocator, pImage)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pImage[0]

def vkGetImageSubresourceLayout(device, image, pSubresource):
	pLayout = ffi.new('VkSubresourceLayout*')
	_callApi(_lib.vkGetImageSubresourceLayout, device, image, pSubresource, pLayout)
	return pLayout[0]

def vkCreateImageView(device, pCreateInfo, pAllocator):
	pView = ffi.new('VkImageView*')
	result = _callApi(_lib.vkCreateImageView, device, pCreateInfo, pAllocator, pView)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pView[0]

def vkCreateShaderModule(device, pCreateInfo, pAllocator):
	pShaderModule = ffi.new('VkShaderModule*')
	result = _callApi(_lib.vkCreateShaderModule, device, pCreateInfo, pAllocator, pShaderModule)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pShaderModule[0]

def vkCreatePipelineCache(device, pCreateInfo, pAllocator):
	pPipelineCache = ffi.new('VkPipelineCache*')
	result = _callApi(_lib.vkCreatePipelineCache, device, pCreateInfo, pAllocator, pPipelineCache)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pPipelineCache[0]

def vkCreatePipelineLayout(device, pCreateInfo, pAllocator):
	pPipelineLayout = ffi.new('VkPipelineLayout*')
	result = _callApi(_lib.vkCreatePipelineLayout, device, pCreateInfo, pAllocator, pPipelineLayout)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pPipelineLayout[0]

def vkCreateSampler(device, pCreateInfo, pAllocator):
	pSampler = ffi.new('VkSampler*')
	result = _callApi(_lib.vkCreateSampler, device, pCreateInfo, pAllocator, pSampler)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pSampler[0]

def vkCreateDescriptorSetLayout(device, pCreateInfo, pAllocator):
	pSetLayout = ffi.new('VkDescriptorSetLayout*')
	result = _callApi(_lib.vkCreateDescriptorSetLayout, device, pCreateInfo, pAllocator, pSetLayout)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pSetLayout[0]

def vkCreateDescriptorPool(device, pCreateInfo, pAllocator):
	pDescriptorPool = ffi.new('VkDescriptorPool*')
	result = _callApi(_lib.vkCreateDescriptorPool, device, pCreateInfo, pAllocator, pDescriptorPool)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pDescriptorPool[0]

def vkCreateFramebuffer(device, pCreateInfo, pAllocator):
	pFramebuffer = ffi.new('VkFramebuffer*')
	result = _callApi(_lib.vkCreateFramebuffer, device, pCreateInfo, pAllocator, pFramebuffer)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pFramebuffer[0]

def vkCreateRenderPass(device, pCreateInfo, pAllocator):
	pRenderPass = ffi.new('VkRenderPass*')
	result = _callApi(_lib.vkCreateRenderPass, device, pCreateInfo, pAllocator, pRenderPass)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pRenderPass[0]

def vkGetRenderAreaGranularity(device, renderPass):
	pGranularity = ffi.new('VkExtent2D*')
	_callApi(_lib.vkGetRenderAreaGranularity, device, renderPass, pGranularity)
	return pGranularity[0]

def vkCreateCommandPool(device, pCreateInfo, pAllocator):
	pCommandPool = ffi.new('VkCommandPool*')
	result = _callApi(_lib.vkCreateCommandPool, device, pCreateInfo, pAllocator, pCommandPool)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pCommandPool[0]

def _wrap_vkCreateAndroidSurfaceKHR(fn):
    def vkCreateAndroidSurfaceKHR(instance, pCreateInfo, pAllocator):
    	pSurface = ffi.new('VkSurfaceKHR*')
    	result = _callApi(fn, instance, pCreateInfo, pAllocator, pSurface)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pSurface[0]
    return vkCreateAndroidSurfaceKHR
def _wrap_vkCreateDisplayModeKHR(fn):
    def vkCreateDisplayModeKHR(physicalDevice, display, pCreateInfo, pAllocator):
    	pMode = ffi.new('VkDisplayModeKHR*')
    	result = _callApi(fn, physicalDevice, display, pCreateInfo, pAllocator, pMode)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pMode[0]
    return vkCreateDisplayModeKHR
def _wrap_vkGetDisplayPlaneCapabilitiesKHR(fn):
    def vkGetDisplayPlaneCapabilitiesKHR(physicalDevice, mode, planeIndex):
    	pCapabilities = ffi.new('VkDisplayPlaneCapabilitiesKHR*')
    	result = _callApi(fn, physicalDevice, mode, planeIndex, pCapabilities)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pCapabilities[0]
    return vkGetDisplayPlaneCapabilitiesKHR
def _wrap_vkCreateDisplayPlaneSurfaceKHR(fn):
    def vkCreateDisplayPlaneSurfaceKHR(instance, pCreateInfo, pAllocator):
    	pSurface = ffi.new('VkSurfaceKHR*')
    	result = _callApi(fn, instance, pCreateInfo, pAllocator, pSurface)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pSurface[0]
    return vkCreateDisplayPlaneSurfaceKHR
def _wrap_vkCreateMirSurfaceKHR(fn):
    def vkCreateMirSurfaceKHR(instance, pCreateInfo, pAllocator):
    	pSurface = ffi.new('VkSurfaceKHR*')
    	result = _callApi(fn, instance, pCreateInfo, pAllocator, pSurface)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pSurface[0]
    return vkCreateMirSurfaceKHR
def _wrap_vkGetPhysicalDeviceMirPresentationSupportKHR(fn):
    def vkGetPhysicalDeviceMirPresentationSupportKHR(physicalDevice, queueFamilyIndex):
    	connection = ffi.new('MirConnection*')
    	_callApi(fn, physicalDevice, queueFamilyIndex, connection)
    	return connection[0]
    return vkGetPhysicalDeviceMirPresentationSupportKHR
def _wrap_vkGetPhysicalDeviceSurfaceSupportKHR(fn):
    def vkGetPhysicalDeviceSurfaceSupportKHR(physicalDevice, queueFamilyIndex, surface):
    	pSupported = ffi.new('VkBool32*')
    	result = _callApi(fn, physicalDevice, queueFamilyIndex, surface, pSupported)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pSupported[0]
    return vkGetPhysicalDeviceSurfaceSupportKHR
def _wrap_vkGetPhysicalDeviceSurfaceCapabilitiesKHR(fn):
    def vkGetPhysicalDeviceSurfaceCapabilitiesKHR(physicalDevice, surface):
    	pSurfaceCapabilities = ffi.new('VkSurfaceCapabilitiesKHR*')
    	result = _callApi(fn, physicalDevice, surface, pSurfaceCapabilities)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pSurfaceCapabilities[0]
    return vkGetPhysicalDeviceSurfaceCapabilitiesKHR
def _wrap_vkCreateSwapchainKHR(fn):
    def vkCreateSwapchainKHR(device, pCreateInfo, pAllocator):
    	pSwapchain = ffi.new('VkSwapchainKHR*')
    	result = _callApi(fn, device, pCreateInfo, pAllocator, pSwapchain)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pSwapchain[0]
    return vkCreateSwapchainKHR
def _wrap_vkAcquireNextImageKHR(fn):
    def vkAcquireNextImageKHR(device, swapchain, timeout, semaphore, fence):
    	pImageIndex = ffi.new('uint32_t*')
    	result = _callApi(fn, device, swapchain, timeout, semaphore, fence, pImageIndex)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pImageIndex[0]
    return vkAcquireNextImageKHR
def _wrap_vkCreateWaylandSurfaceKHR(fn):
    def vkCreateWaylandSurfaceKHR(instance, pCreateInfo, pAllocator):
    	pSurface = ffi.new('VkSurfaceKHR*')
    	result = _callApi(fn, instance, pCreateInfo, pAllocator, pSurface)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pSurface[0]
    return vkCreateWaylandSurfaceKHR
def _wrap_vkGetPhysicalDeviceWaylandPresentationSupportKHR(fn):
    def vkGetPhysicalDeviceWaylandPresentationSupportKHR(physicalDevice, queueFamilyIndex):
    	display = ffi.new('struct wl_display*')
    	_callApi(fn, physicalDevice, queueFamilyIndex, display)
    	return display[0]
    return vkGetPhysicalDeviceWaylandPresentationSupportKHR
def _wrap_vkCreateWin32SurfaceKHR(fn):
    def vkCreateWin32SurfaceKHR(instance, pCreateInfo, pAllocator):
    	pSurface = ffi.new('VkSurfaceKHR*')
    	result = _callApi(fn, instance, pCreateInfo, pAllocator, pSurface)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pSurface[0]
    return vkCreateWin32SurfaceKHR
def _wrap_vkCreateXlibSurfaceKHR(fn):
    def vkCreateXlibSurfaceKHR(instance, pCreateInfo, pAllocator):
    	pSurface = ffi.new('VkSurfaceKHR*')
    	result = _callApi(fn, instance, pCreateInfo, pAllocator, pSurface)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pSurface[0]
    return vkCreateXlibSurfaceKHR
def _wrap_vkCreateXcbSurfaceKHR(fn):
    def vkCreateXcbSurfaceKHR(instance, pCreateInfo, pAllocator):
    	pSurface = ffi.new('VkSurfaceKHR*')
    	result = _callApi(fn, instance, pCreateInfo, pAllocator, pSurface)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pSurface[0]
    return vkCreateXcbSurfaceKHR
def _wrap_vkCreateDebugReportCallbackEXT(fn):
    def vkCreateDebugReportCallbackEXT(instance, pCreateInfo, pAllocator):
    	pCallback = ffi.new('VkDebugReportCallbackEXT*')
    	result = _callApi(fn, instance, pCreateInfo, pAllocator, pCallback)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pCallback[0]
    return vkCreateDebugReportCallbackEXT
def _wrap_vkGetPhysicalDeviceExternalImageFormatPropertiesNV(fn):
    def vkGetPhysicalDeviceExternalImageFormatPropertiesNV(physicalDevice, format, type, tiling, usage, flags, externalHandleType):
    	pExternalImageFormatProperties = ffi.new('VkExternalImageFormatPropertiesNV*')
    	result = _callApi(fn, physicalDevice, format, type, tiling, usage, flags, externalHandleType, pExternalImageFormatProperties)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pExternalImageFormatProperties[0]
    return vkGetPhysicalDeviceExternalImageFormatPropertiesNV
def _wrap_vkGetMemoryWin32HandleNV(fn):
    def vkGetMemoryWin32HandleNV(device, memory, handleType):
    	pHandle = ffi.new('HANDLE*')
    	result = _callApi(fn, device, memory, handleType, pHandle)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pHandle[0]
    return vkGetMemoryWin32HandleNV

def vkEnumeratePhysicalDevices(instance):
	pPhysicalDeviceCount = ffi.new('uint32_t*')
	result = _callApi(_lib.vkEnumeratePhysicalDevices, instance, pPhysicalDeviceCount, ffi.NULL)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	pPhysicalDevices = ffi.new('VkPhysicalDevice[]', pPhysicalDeviceCount[0])
	result = _callApi(_lib.vkEnumeratePhysicalDevices, instance, pPhysicalDeviceCount, pPhysicalDevices)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pPhysicalDevices

def vkGetPhysicalDeviceQueueFamilyProperties(physicalDevice):
	pQueueFamilyPropertyCount = ffi.new('uint32_t*')
	_callApi(_lib.vkGetPhysicalDeviceQueueFamilyProperties, physicalDevice, pQueueFamilyPropertyCount, ffi.NULL)
	pQueueFamilyProperties = ffi.new('VkQueueFamilyProperties[]', pQueueFamilyPropertyCount[0])
	_callApi(_lib.vkGetPhysicalDeviceQueueFamilyProperties, physicalDevice, pQueueFamilyPropertyCount, pQueueFamilyProperties)
	return pQueueFamilyProperties

def vkEnumerateInstanceLayerProperties():
	pPropertyCount = ffi.new('uint32_t*')
	result = _callApi(_lib.vkEnumerateInstanceLayerProperties, pPropertyCount, ffi.NULL)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	pProperties = ffi.new('VkLayerProperties[]', pPropertyCount[0])
	result = _callApi(_lib.vkEnumerateInstanceLayerProperties, pPropertyCount, pProperties)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pProperties

def vkEnumerateInstanceExtensionProperties(pLayerName):
	pPropertyCount = ffi.new('uint32_t*')
	result = _callApi(_lib.vkEnumerateInstanceExtensionProperties, pLayerName, pPropertyCount, ffi.NULL)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	pProperties = ffi.new('VkExtensionProperties[]', pPropertyCount[0])
	result = _callApi(_lib.vkEnumerateInstanceExtensionProperties, pLayerName, pPropertyCount, pProperties)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pProperties

def vkEnumerateDeviceLayerProperties(physicalDevice):
	pPropertyCount = ffi.new('uint32_t*')
	result = _callApi(_lib.vkEnumerateDeviceLayerProperties, physicalDevice, pPropertyCount, ffi.NULL)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	pProperties = ffi.new('VkLayerProperties[]', pPropertyCount[0])
	result = _callApi(_lib.vkEnumerateDeviceLayerProperties, physicalDevice, pPropertyCount, pProperties)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pProperties

def vkEnumerateDeviceExtensionProperties(physicalDevice, pLayerName):
	pPropertyCount = ffi.new('uint32_t*')
	result = _callApi(_lib.vkEnumerateDeviceExtensionProperties, physicalDevice, pLayerName, pPropertyCount, ffi.NULL)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	pProperties = ffi.new('VkExtensionProperties[]', pPropertyCount[0])
	result = _callApi(_lib.vkEnumerateDeviceExtensionProperties, physicalDevice, pLayerName, pPropertyCount, pProperties)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pProperties

def vkGetImageSparseMemoryRequirements(device, image):
	pSparseMemoryRequirementCount = ffi.new('uint32_t*')
	_callApi(_lib.vkGetImageSparseMemoryRequirements, device, image, pSparseMemoryRequirementCount, ffi.NULL)
	pSparseMemoryRequirements = ffi.new('VkSparseImageMemoryRequirements[]', pSparseMemoryRequirementCount[0])
	_callApi(_lib.vkGetImageSparseMemoryRequirements, device, image, pSparseMemoryRequirementCount, pSparseMemoryRequirements)
	return pSparseMemoryRequirements

def vkGetPhysicalDeviceSparseImageFormatProperties(physicalDevice, format, type, samples, usage, tiling):
	pPropertyCount = ffi.new('uint32_t*')
	_callApi(_lib.vkGetPhysicalDeviceSparseImageFormatProperties, physicalDevice, format, type, samples, usage, tiling, pPropertyCount, ffi.NULL)
	pProperties = ffi.new('VkSparseImageFormatProperties[]', pPropertyCount[0])
	_callApi(_lib.vkGetPhysicalDeviceSparseImageFormatProperties, physicalDevice, format, type, samples, usage, tiling, pPropertyCount, pProperties)
	return pProperties

def vkGetPipelineCacheData(device, pipelineCache):
	pDataSize = ffi.new('size_t*')
	result = _callApi(_lib.vkGetPipelineCacheData, device, pipelineCache, pDataSize, ffi.NULL)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	pData = ffi.new('void[]', pDataSize[0])
	result = _callApi(_lib.vkGetPipelineCacheData, device, pipelineCache, pDataSize, pData)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pData

def _wrap_vkGetPhysicalDeviceDisplayPropertiesKHR(fn):
    def vkGetPhysicalDeviceDisplayPropertiesKHR(physicalDevice):
    	pPropertyCount = ffi.new('uint32_t*')
    	result = _callApi(fn, physicalDevice, pPropertyCount, ffi.NULL)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	pProperties = ffi.new('VkDisplayPropertiesKHR[]', pPropertyCount[0])
    	result = _callApi(fn, physicalDevice, pPropertyCount, pProperties)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pProperties
    return vkGetPhysicalDeviceDisplayPropertiesKHR
def _wrap_vkGetPhysicalDeviceDisplayPlanePropertiesKHR(fn):
    def vkGetPhysicalDeviceDisplayPlanePropertiesKHR(physicalDevice):
    	pPropertyCount = ffi.new('uint32_t*')
    	result = _callApi(fn, physicalDevice, pPropertyCount, ffi.NULL)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	pProperties = ffi.new('VkDisplayPlanePropertiesKHR[]', pPropertyCount[0])
    	result = _callApi(fn, physicalDevice, pPropertyCount, pProperties)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pProperties
    return vkGetPhysicalDeviceDisplayPlanePropertiesKHR
def _wrap_vkGetDisplayPlaneSupportedDisplaysKHR(fn):
    def vkGetDisplayPlaneSupportedDisplaysKHR(physicalDevice, planeIndex):
    	pDisplayCount = ffi.new('uint32_t*')
    	result = _callApi(fn, physicalDevice, planeIndex, pDisplayCount, ffi.NULL)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	pDisplays = ffi.new('VkDisplayKHR[]', pDisplayCount[0])
    	result = _callApi(fn, physicalDevice, planeIndex, pDisplayCount, pDisplays)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pDisplays
    return vkGetDisplayPlaneSupportedDisplaysKHR
def _wrap_vkGetDisplayModePropertiesKHR(fn):
    def vkGetDisplayModePropertiesKHR(physicalDevice, display):
    	pPropertyCount = ffi.new('uint32_t*')
    	result = _callApi(fn, physicalDevice, display, pPropertyCount, ffi.NULL)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	pProperties = ffi.new('VkDisplayModePropertiesKHR[]', pPropertyCount[0])
    	result = _callApi(fn, physicalDevice, display, pPropertyCount, pProperties)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pProperties
    return vkGetDisplayModePropertiesKHR
def _wrap_vkGetPhysicalDeviceSurfaceFormatsKHR(fn):
    def vkGetPhysicalDeviceSurfaceFormatsKHR(physicalDevice, surface):
    	pSurfaceFormatCount = ffi.new('uint32_t*')
    	result = _callApi(fn, physicalDevice, surface, pSurfaceFormatCount, ffi.NULL)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	pSurfaceFormats = ffi.new('VkSurfaceFormatKHR[]', pSurfaceFormatCount[0])
    	result = _callApi(fn, physicalDevice, surface, pSurfaceFormatCount, pSurfaceFormats)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pSurfaceFormats
    return vkGetPhysicalDeviceSurfaceFormatsKHR
def _wrap_vkGetPhysicalDeviceSurfacePresentModesKHR(fn):
    def vkGetPhysicalDeviceSurfacePresentModesKHR(physicalDevice, surface):
    	pPresentModeCount = ffi.new('uint32_t*')
    	result = _callApi(fn, physicalDevice, surface, pPresentModeCount, ffi.NULL)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	pPresentModes = ffi.new('VkPresentModeKHR[]', pPresentModeCount[0])
    	result = _callApi(fn, physicalDevice, surface, pPresentModeCount, pPresentModes)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pPresentModes
    return vkGetPhysicalDeviceSurfacePresentModesKHR
def _wrap_vkGetSwapchainImagesKHR(fn):
    def vkGetSwapchainImagesKHR(device, swapchain):
    	pSwapchainImageCount = ffi.new('uint32_t*')
    	result = _callApi(fn, device, swapchain, pSwapchainImageCount, ffi.NULL)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	pSwapchainImages = ffi.new('VkImage[]', pSwapchainImageCount[0])
    	result = _callApi(fn, device, swapchain, pSwapchainImageCount, pSwapchainImages)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pSwapchainImages
    return vkGetSwapchainImagesKHR

def vkCreateGraphicsPipelines(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator):
	pPipelines = ffi.new('VkPipeline*')
	result = _callApi(_lib.vkCreateGraphicsPipelines, device, pipelineCache, createInfoCount, pCreateInfos, pAllocator, pPipelines)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pPipelines

def vkCreateComputePipelines(device, pipelineCache, createInfoCount, pCreateInfos, pAllocator):
	pPipelines = ffi.new('VkPipeline*')
	result = _callApi(_lib.vkCreateComputePipelines, device, pipelineCache, createInfoCount, pCreateInfos, pAllocator, pPipelines)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pPipelines

def vkAllocateDescriptorSets(device, pAllocateInfo):
	pDescriptorSets = ffi.new('VkDescriptorSet*')
	result = _callApi(_lib.vkAllocateDescriptorSets, device, pAllocateInfo, pDescriptorSets)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pDescriptorSets

def vkAllocateCommandBuffers(device, pAllocateInfo):
	pCommandBuffers = ffi.new('VkCommandBuffer*')
	result = _callApi(_lib.vkAllocateCommandBuffers, device, pAllocateInfo, pCommandBuffers)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]
	return pCommandBuffers

def _wrap_vkCreateSharedSwapchainsKHR(fn):
    def vkCreateSharedSwapchainsKHR(device, swapchainCount, pCreateInfos, pAllocator):
    	pSwapchains = ffi.new('VkSwapchainKHR*')
    	result = _callApi(fn, device, swapchainCount, pCreateInfos, pAllocator, pSwapchains)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    	return pSwapchains
    return vkCreateSharedSwapchainsKHR

def vkDestroyInstance(instance, pAllocator):
	_callApi(_lib.vkDestroyInstance, instance, pAllocator)

def vkDestroyDevice(device, pAllocator):
	_callApi(_lib.vkDestroyDevice, device, pAllocator)

def vkQueueSubmit(queue, submitCount, pSubmits, fence):
	result = _callApi(_lib.vkQueueSubmit, queue, submitCount, pSubmits, fence)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkQueueWaitIdle(queue):
	result = _callApi(_lib.vkQueueWaitIdle, queue)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkDeviceWaitIdle(device):
	result = _callApi(_lib.vkDeviceWaitIdle, device)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkFreeMemory(device, memory, pAllocator):
	_callApi(_lib.vkFreeMemory, device, memory, pAllocator)

def vkUnmapMemory(device, memory):
	_callApi(_lib.vkUnmapMemory, device, memory)

def vkFlushMappedMemoryRanges(device, memoryRangeCount, pMemoryRanges):
	result = _callApi(_lib.vkFlushMappedMemoryRanges, device, memoryRangeCount, pMemoryRanges)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkInvalidateMappedMemoryRanges(device, memoryRangeCount, pMemoryRanges):
	result = _callApi(_lib.vkInvalidateMappedMemoryRanges, device, memoryRangeCount, pMemoryRanges)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkBindBufferMemory(device, buffer, memory, memoryOffset):
	result = _callApi(_lib.vkBindBufferMemory, device, buffer, memory, memoryOffset)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkBindImageMemory(device, image, memory, memoryOffset):
	result = _callApi(_lib.vkBindImageMemory, device, image, memory, memoryOffset)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkQueueBindSparse(queue, bindInfoCount, pBindInfo, fence):
	result = _callApi(_lib.vkQueueBindSparse, queue, bindInfoCount, pBindInfo, fence)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkDestroyFence(device, fence, pAllocator):
	_callApi(_lib.vkDestroyFence, device, fence, pAllocator)

def vkResetFences(device, fenceCount, pFences):
	result = _callApi(_lib.vkResetFences, device, fenceCount, pFences)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkGetFenceStatus(device, fence):
	result = _callApi(_lib.vkGetFenceStatus, device, fence)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkWaitForFences(device, fenceCount, pFences, waitAll, timeout):
	result = _callApi(_lib.vkWaitForFences, device, fenceCount, pFences, waitAll, timeout)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkDestroySemaphore(device, semaphore, pAllocator):
	_callApi(_lib.vkDestroySemaphore, device, semaphore, pAllocator)

def vkDestroyEvent(device, event, pAllocator):
	_callApi(_lib.vkDestroyEvent, device, event, pAllocator)

def vkGetEventStatus(device, event):
	result = _callApi(_lib.vkGetEventStatus, device, event)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkSetEvent(device, event):
	result = _callApi(_lib.vkSetEvent, device, event)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkResetEvent(device, event):
	result = _callApi(_lib.vkResetEvent, device, event)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkDestroyQueryPool(device, queryPool, pAllocator):
	_callApi(_lib.vkDestroyQueryPool, device, queryPool, pAllocator)

def vkGetQueryPoolResults(device, queryPool, firstQuery, queryCount, dataSize, pData, stride, flags):
	result = _callApi(_lib.vkGetQueryPoolResults, device, queryPool, firstQuery, queryCount, dataSize, pData, stride, flags)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkDestroyBuffer(device, buffer, pAllocator):
	_callApi(_lib.vkDestroyBuffer, device, buffer, pAllocator)

def vkDestroyBufferView(device, bufferView, pAllocator):
	_callApi(_lib.vkDestroyBufferView, device, bufferView, pAllocator)

def vkDestroyImage(device, image, pAllocator):
	_callApi(_lib.vkDestroyImage, device, image, pAllocator)

def vkDestroyImageView(device, imageView, pAllocator):
	_callApi(_lib.vkDestroyImageView, device, imageView, pAllocator)

def vkDestroyShaderModule(device, shaderModule, pAllocator):
	_callApi(_lib.vkDestroyShaderModule, device, shaderModule, pAllocator)

def vkDestroyPipelineCache(device, pipelineCache, pAllocator):
	_callApi(_lib.vkDestroyPipelineCache, device, pipelineCache, pAllocator)

def vkMergePipelineCaches(device, dstCache, srcCacheCount, pSrcCaches):
	result = _callApi(_lib.vkMergePipelineCaches, device, dstCache, srcCacheCount, pSrcCaches)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkDestroyPipeline(device, pipeline, pAllocator):
	_callApi(_lib.vkDestroyPipeline, device, pipeline, pAllocator)

def vkDestroyPipelineLayout(device, pipelineLayout, pAllocator):
	_callApi(_lib.vkDestroyPipelineLayout, device, pipelineLayout, pAllocator)

def vkDestroySampler(device, sampler, pAllocator):
	_callApi(_lib.vkDestroySampler, device, sampler, pAllocator)

def vkDestroyDescriptorSetLayout(device, descriptorSetLayout, pAllocator):
	_callApi(_lib.vkDestroyDescriptorSetLayout, device, descriptorSetLayout, pAllocator)

def vkDestroyDescriptorPool(device, descriptorPool, pAllocator):
	_callApi(_lib.vkDestroyDescriptorPool, device, descriptorPool, pAllocator)

def vkResetDescriptorPool(device, descriptorPool, flags):
	result = _callApi(_lib.vkResetDescriptorPool, device, descriptorPool, flags)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkFreeDescriptorSets(device, descriptorPool, descriptorSetCount, pDescriptorSets):
	result = _callApi(_lib.vkFreeDescriptorSets, device, descriptorPool, descriptorSetCount, pDescriptorSets)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkUpdateDescriptorSets(device, descriptorWriteCount, pDescriptorWrites, descriptorCopyCount, pDescriptorCopies):
	_callApi(_lib.vkUpdateDescriptorSets, device, descriptorWriteCount, pDescriptorWrites, descriptorCopyCount, pDescriptorCopies)

def vkDestroyFramebuffer(device, framebuffer, pAllocator):
	_callApi(_lib.vkDestroyFramebuffer, device, framebuffer, pAllocator)

def vkDestroyRenderPass(device, renderPass, pAllocator):
	_callApi(_lib.vkDestroyRenderPass, device, renderPass, pAllocator)

def vkDestroyCommandPool(device, commandPool, pAllocator):
	_callApi(_lib.vkDestroyCommandPool, device, commandPool, pAllocator)

def vkResetCommandPool(device, commandPool, flags):
	result = _callApi(_lib.vkResetCommandPool, device, commandPool, flags)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkFreeCommandBuffers(device, commandPool, commandBufferCount, pCommandBuffers):
	_callApi(_lib.vkFreeCommandBuffers, device, commandPool, commandBufferCount, pCommandBuffers)

def vkBeginCommandBuffer(commandBuffer, pBeginInfo):
	result = _callApi(_lib.vkBeginCommandBuffer, commandBuffer, pBeginInfo)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkEndCommandBuffer(commandBuffer):
	result = _callApi(_lib.vkEndCommandBuffer, commandBuffer)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkResetCommandBuffer(commandBuffer, flags):
	result = _callApi(_lib.vkResetCommandBuffer, commandBuffer, flags)
	if result!=VK_SUCCESS:
		raise _exception_codes[result]

def vkCmdBindPipeline(commandBuffer, pipelineBindPoint, pipeline):
	_callApi(_lib.vkCmdBindPipeline, commandBuffer, pipelineBindPoint, pipeline)

def vkCmdSetViewport(commandBuffer, firstViewport, viewportCount, pViewports):
	_callApi(_lib.vkCmdSetViewport, commandBuffer, firstViewport, viewportCount, pViewports)

def vkCmdSetScissor(commandBuffer, firstScissor, scissorCount, pScissors):
	_callApi(_lib.vkCmdSetScissor, commandBuffer, firstScissor, scissorCount, pScissors)

def vkCmdSetLineWidth(commandBuffer, lineWidth):
	_callApi(_lib.vkCmdSetLineWidth, commandBuffer, lineWidth)

def vkCmdSetDepthBias(commandBuffer, depthBiasConstantFactor, depthBiasClamp, depthBiasSlopeFactor):
	_callApi(_lib.vkCmdSetDepthBias, commandBuffer, depthBiasConstantFactor, depthBiasClamp, depthBiasSlopeFactor)

def vkCmdSetBlendConstants(commandBuffer, blendConstants):
	_callApi(_lib.vkCmdSetBlendConstants, commandBuffer, blendConstants)

def vkCmdSetDepthBounds(commandBuffer, minDepthBounds, maxDepthBounds):
	_callApi(_lib.vkCmdSetDepthBounds, commandBuffer, minDepthBounds, maxDepthBounds)

def vkCmdSetStencilCompareMask(commandBuffer, faceMask, compareMask):
	_callApi(_lib.vkCmdSetStencilCompareMask, commandBuffer, faceMask, compareMask)

def vkCmdSetStencilWriteMask(commandBuffer, faceMask, writeMask):
	_callApi(_lib.vkCmdSetStencilWriteMask, commandBuffer, faceMask, writeMask)

def vkCmdSetStencilReference(commandBuffer, faceMask, reference):
	_callApi(_lib.vkCmdSetStencilReference, commandBuffer, faceMask, reference)

def vkCmdBindDescriptorSets(commandBuffer, pipelineBindPoint, layout, firstSet, descriptorSetCount, pDescriptorSets, dynamicOffsetCount, pDynamicOffsets):
	_callApi(_lib.vkCmdBindDescriptorSets, commandBuffer, pipelineBindPoint, layout, firstSet, descriptorSetCount, pDescriptorSets, dynamicOffsetCount, pDynamicOffsets)

def vkCmdBindIndexBuffer(commandBuffer, buffer, offset, indexType):
	_callApi(_lib.vkCmdBindIndexBuffer, commandBuffer, buffer, offset, indexType)

def vkCmdBindVertexBuffers(commandBuffer, firstBinding, bindingCount, pBuffers, pOffsets):
	_callApi(_lib.vkCmdBindVertexBuffers, commandBuffer, firstBinding, bindingCount, pBuffers, pOffsets)

def vkCmdDraw(commandBuffer, vertexCount, instanceCount, firstVertex, firstInstance):
	_callApi(_lib.vkCmdDraw, commandBuffer, vertexCount, instanceCount, firstVertex, firstInstance)

def vkCmdDrawIndexed(commandBuffer, indexCount, instanceCount, firstIndex, vertexOffset, firstInstance):
	_callApi(_lib.vkCmdDrawIndexed, commandBuffer, indexCount, instanceCount, firstIndex, vertexOffset, firstInstance)

def vkCmdDrawIndirect(commandBuffer, buffer, offset, drawCount, stride):
	_callApi(_lib.vkCmdDrawIndirect, commandBuffer, buffer, offset, drawCount, stride)

def vkCmdDrawIndexedIndirect(commandBuffer, buffer, offset, drawCount, stride):
	_callApi(_lib.vkCmdDrawIndexedIndirect, commandBuffer, buffer, offset, drawCount, stride)

def vkCmdDispatch(commandBuffer, x, y, z):
	_callApi(_lib.vkCmdDispatch, commandBuffer, x, y, z)

def vkCmdDispatchIndirect(commandBuffer, buffer, offset):
	_callApi(_lib.vkCmdDispatchIndirect, commandBuffer, buffer, offset)

def vkCmdCopyBuffer(commandBuffer, srcBuffer, dstBuffer, regionCount, pRegions):
	_callApi(_lib.vkCmdCopyBuffer, commandBuffer, srcBuffer, dstBuffer, regionCount, pRegions)

def vkCmdCopyImage(commandBuffer, srcImage, srcImageLayout, dstImage, dstImageLayout, regionCount, pRegions):
	_callApi(_lib.vkCmdCopyImage, commandBuffer, srcImage, srcImageLayout, dstImage, dstImageLayout, regionCount, pRegions)

def vkCmdBlitImage(commandBuffer, srcImage, srcImageLayout, dstImage, dstImageLayout, regionCount, pRegions, filter):
	_callApi(_lib.vkCmdBlitImage, commandBuffer, srcImage, srcImageLayout, dstImage, dstImageLayout, regionCount, pRegions, filter)

def vkCmdCopyBufferToImage(commandBuffer, srcBuffer, dstImage, dstImageLayout, regionCount, pRegions):
	_callApi(_lib.vkCmdCopyBufferToImage, commandBuffer, srcBuffer, dstImage, dstImageLayout, regionCount, pRegions)

def vkCmdCopyImageToBuffer(commandBuffer, srcImage, srcImageLayout, dstBuffer, regionCount, pRegions):
	_callApi(_lib.vkCmdCopyImageToBuffer, commandBuffer, srcImage, srcImageLayout, dstBuffer, regionCount, pRegions)

def vkCmdUpdateBuffer(commandBuffer, dstBuffer, dstOffset, dataSize, pData):
	_callApi(_lib.vkCmdUpdateBuffer, commandBuffer, dstBuffer, dstOffset, dataSize, pData)

def vkCmdFillBuffer(commandBuffer, dstBuffer, dstOffset, size, data):
	_callApi(_lib.vkCmdFillBuffer, commandBuffer, dstBuffer, dstOffset, size, data)

def vkCmdClearColorImage(commandBuffer, image, imageLayout, pColor, rangeCount, pRanges):
	_callApi(_lib.vkCmdClearColorImage, commandBuffer, image, imageLayout, pColor, rangeCount, pRanges)

def vkCmdClearDepthStencilImage(commandBuffer, image, imageLayout, pDepthStencil, rangeCount, pRanges):
	_callApi(_lib.vkCmdClearDepthStencilImage, commandBuffer, image, imageLayout, pDepthStencil, rangeCount, pRanges)

def vkCmdClearAttachments(commandBuffer, attachmentCount, pAttachments, rectCount, pRects):
	_callApi(_lib.vkCmdClearAttachments, commandBuffer, attachmentCount, pAttachments, rectCount, pRects)

def vkCmdResolveImage(commandBuffer, srcImage, srcImageLayout, dstImage, dstImageLayout, regionCount, pRegions):
	_callApi(_lib.vkCmdResolveImage, commandBuffer, srcImage, srcImageLayout, dstImage, dstImageLayout, regionCount, pRegions)

def vkCmdSetEvent(commandBuffer, event, stageMask):
	_callApi(_lib.vkCmdSetEvent, commandBuffer, event, stageMask)

def vkCmdResetEvent(commandBuffer, event, stageMask):
	_callApi(_lib.vkCmdResetEvent, commandBuffer, event, stageMask)

def vkCmdWaitEvents(commandBuffer, eventCount, pEvents, srcStageMask, dstStageMask, memoryBarrierCount, pMemoryBarriers, bufferMemoryBarrierCount, pBufferMemoryBarriers, imageMemoryBarrierCount, pImageMemoryBarriers):
	_callApi(_lib.vkCmdWaitEvents, commandBuffer, eventCount, pEvents, srcStageMask, dstStageMask, memoryBarrierCount, pMemoryBarriers, bufferMemoryBarrierCount, pBufferMemoryBarriers, imageMemoryBarrierCount, pImageMemoryBarriers)

def vkCmdPipelineBarrier(commandBuffer, srcStageMask, dstStageMask, dependencyFlags, memoryBarrierCount, pMemoryBarriers, bufferMemoryBarrierCount, pBufferMemoryBarriers, imageMemoryBarrierCount, pImageMemoryBarriers):
	_callApi(_lib.vkCmdPipelineBarrier, commandBuffer, srcStageMask, dstStageMask, dependencyFlags, memoryBarrierCount, pMemoryBarriers, bufferMemoryBarrierCount, pBufferMemoryBarriers, imageMemoryBarrierCount, pImageMemoryBarriers)

def vkCmdBeginQuery(commandBuffer, queryPool, query, flags):
	_callApi(_lib.vkCmdBeginQuery, commandBuffer, queryPool, query, flags)

def vkCmdEndQuery(commandBuffer, queryPool, query):
	_callApi(_lib.vkCmdEndQuery, commandBuffer, queryPool, query)

def vkCmdResetQueryPool(commandBuffer, queryPool, firstQuery, queryCount):
	_callApi(_lib.vkCmdResetQueryPool, commandBuffer, queryPool, firstQuery, queryCount)

def vkCmdWriteTimestamp(commandBuffer, pipelineStage, queryPool, query):
	_callApi(_lib.vkCmdWriteTimestamp, commandBuffer, pipelineStage, queryPool, query)

def vkCmdCopyQueryPoolResults(commandBuffer, queryPool, firstQuery, queryCount, dstBuffer, dstOffset, stride, flags):
	_callApi(_lib.vkCmdCopyQueryPoolResults, commandBuffer, queryPool, firstQuery, queryCount, dstBuffer, dstOffset, stride, flags)

def vkCmdPushConstants(commandBuffer, layout, stageFlags, offset, size, pValues):
	_callApi(_lib.vkCmdPushConstants, commandBuffer, layout, stageFlags, offset, size, pValues)

def vkCmdBeginRenderPass(commandBuffer, pRenderPassBegin, contents):
	_callApi(_lib.vkCmdBeginRenderPass, commandBuffer, pRenderPassBegin, contents)

def vkCmdNextSubpass(commandBuffer, contents):
	_callApi(_lib.vkCmdNextSubpass, commandBuffer, contents)

def vkCmdEndRenderPass(commandBuffer):
	_callApi(_lib.vkCmdEndRenderPass, commandBuffer)

def vkCmdExecuteCommands(commandBuffer, commandBufferCount, pCommandBuffers):
	_callApi(_lib.vkCmdExecuteCommands, commandBuffer, commandBufferCount, pCommandBuffers)

def _wrap_vkDestroySurfaceKHR(fn):
    def vkDestroySurfaceKHR(instance, surface, pAllocator):
    	_callApi(fn, instance, surface, pAllocator)
    return vkDestroySurfaceKHR
def _wrap_vkDestroySwapchainKHR(fn):
    def vkDestroySwapchainKHR(device, swapchain, pAllocator):
    	_callApi(fn, device, swapchain, pAllocator)
    return vkDestroySwapchainKHR
def _wrap_vkQueuePresentKHR(fn):
    def vkQueuePresentKHR(queue, pPresentInfo):
    	result = _callApi(fn, queue, pPresentInfo)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    return vkQueuePresentKHR
def _wrap_vkGetPhysicalDeviceWin32PresentationSupportKHR(fn):
    def vkGetPhysicalDeviceWin32PresentationSupportKHR(physicalDevice, queueFamilyIndex):
    	_callApi(fn, physicalDevice, queueFamilyIndex)
    return vkGetPhysicalDeviceWin32PresentationSupportKHR
def _wrap_vkGetPhysicalDeviceXlibPresentationSupportKHR(fn):
    def vkGetPhysicalDeviceXlibPresentationSupportKHR(physicalDevice, queueFamilyIndex, dpy, visualID):
    	_callApi(fn, physicalDevice, queueFamilyIndex, dpy, visualID)
    return vkGetPhysicalDeviceXlibPresentationSupportKHR
def _wrap_vkGetPhysicalDeviceXcbPresentationSupportKHR(fn):
    def vkGetPhysicalDeviceXcbPresentationSupportKHR(physicalDevice, queueFamilyIndex, connection, visual_id):
    	_callApi(fn, physicalDevice, queueFamilyIndex, connection, visual_id)
    return vkGetPhysicalDeviceXcbPresentationSupportKHR
def _wrap_vkDestroyDebugReportCallbackEXT(fn):
    def vkDestroyDebugReportCallbackEXT(instance, callback, pAllocator):
    	_callApi(fn, instance, callback, pAllocator)
    return vkDestroyDebugReportCallbackEXT
def _wrap_vkDebugReportMessageEXT(fn):
    def vkDebugReportMessageEXT(instance, flags, objectType, object, location, messageCode, pLayerPrefix, pMessage):
    	_callApi(fn, instance, flags, objectType, object, location, messageCode, pLayerPrefix, pMessage)
    return vkDebugReportMessageEXT
def _wrap_vkDebugMarkerSetObjectNameEXT(fn):
    def vkDebugMarkerSetObjectNameEXT(device, pNameInfo):
    	result = _callApi(fn, device, pNameInfo)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    return vkDebugMarkerSetObjectNameEXT
def _wrap_vkDebugMarkerSetObjectTagEXT(fn):
    def vkDebugMarkerSetObjectTagEXT(device, pTagInfo):
    	result = _callApi(fn, device, pTagInfo)
    	if result!=VK_SUCCESS:
    		raise _exception_codes[result]
    return vkDebugMarkerSetObjectTagEXT
def _wrap_vkCmdDebugMarkerBeginEXT(fn):
    def vkCmdDebugMarkerBeginEXT(commandBuffer, pMarkerInfo):
    	_callApi(fn, commandBuffer, pMarkerInfo)
    return vkCmdDebugMarkerBeginEXT
def _wrap_vkCmdDebugMarkerEndEXT(fn):
    def vkCmdDebugMarkerEndEXT(commandBuffer):
    	_callApi(fn, commandBuffer)
    return vkCmdDebugMarkerEndEXT
def _wrap_vkCmdDebugMarkerInsertEXT(fn):
    def vkCmdDebugMarkerInsertEXT(commandBuffer, pMarkerInfo):
    	_callApi(fn, commandBuffer, pMarkerInfo)
    return vkCmdDebugMarkerInsertEXT
def _wrap_vkCmdDrawIndirectCountAMD(fn):
    def vkCmdDrawIndirectCountAMD(commandBuffer, buffer, offset, countBuffer, countBufferOffset, maxDrawCount, stride):
    	_callApi(fn, commandBuffer, buffer, offset, countBuffer, countBufferOffset, maxDrawCount, stride)
    return vkCmdDrawIndirectCountAMD
def _wrap_vkCmdDrawIndexedIndirectCountAMD(fn):
    def vkCmdDrawIndexedIndirectCountAMD(commandBuffer, buffer, offset, countBuffer, countBufferOffset, maxDrawCount, stride):
    	_callApi(fn, commandBuffer, buffer, offset, countBuffer, countBufferOffset, maxDrawCount, stride)
    return vkCmdDrawIndexedIndirectCountAMD


_instance_ext_funcs = {
	'vkCreateXlibSurfaceKHR':_wrap_vkCreateXlibSurfaceKHR,
	'vkGetPhysicalDeviceXlibPresentationSupportKHR':_wrap_vkGetPhysicalDeviceXlibPresentationSupportKHR,
	'vkCreateXcbSurfaceKHR':_wrap_vkCreateXcbSurfaceKHR,
	'vkGetPhysicalDeviceXcbPresentationSupportKHR':_wrap_vkGetPhysicalDeviceXcbPresentationSupportKHR,
	'vkCreateWaylandSurfaceKHR':_wrap_vkCreateWaylandSurfaceKHR,
	'vkGetPhysicalDeviceWaylandPresentationSupportKHR':_wrap_vkGetPhysicalDeviceWaylandPresentationSupportKHR,
	'vkCreateMirSurfaceKHR':_wrap_vkCreateMirSurfaceKHR,
	'vkGetPhysicalDeviceMirPresentationSupportKHR':_wrap_vkGetPhysicalDeviceMirPresentationSupportKHR,
	'vkCreateWin32SurfaceKHR':_wrap_vkCreateWin32SurfaceKHR,
	'vkGetPhysicalDeviceWin32PresentationSupportKHR':_wrap_vkGetPhysicalDeviceWin32PresentationSupportKHR,
	'vkCreateAndroidSurfaceKHR':_wrap_vkCreateAndroidSurfaceKHR,
	'vkDestroySurfaceKHR':_wrap_vkDestroySurfaceKHR,
	'vkGetPhysicalDeviceSurfaceSupportKHR':_wrap_vkGetPhysicalDeviceSurfaceSupportKHR,
	'vkGetPhysicalDeviceSurfaceCapabilitiesKHR':_wrap_vkGetPhysicalDeviceSurfaceCapabilitiesKHR,
	'vkGetPhysicalDeviceSurfaceFormatsKHR':_wrap_vkGetPhysicalDeviceSurfaceFormatsKHR,
	'vkGetPhysicalDeviceSurfacePresentModesKHR':_wrap_vkGetPhysicalDeviceSurfacePresentModesKHR,
	'vkGetPhysicalDeviceDisplayPropertiesKHR':_wrap_vkGetPhysicalDeviceDisplayPropertiesKHR,
	'vkGetPhysicalDeviceDisplayPlanePropertiesKHR':_wrap_vkGetPhysicalDeviceDisplayPlanePropertiesKHR,
	'vkGetDisplayPlaneSupportedDisplaysKHR':_wrap_vkGetDisplayPlaneSupportedDisplaysKHR,
	'vkGetDisplayModePropertiesKHR':_wrap_vkGetDisplayModePropertiesKHR,
	'vkCreateDisplayModeKHR':_wrap_vkCreateDisplayModeKHR,
	'vkGetDisplayPlaneCapabilitiesKHR':_wrap_vkGetDisplayPlaneCapabilitiesKHR,
	'vkCreateDisplayPlaneSurfaceKHR':_wrap_vkCreateDisplayPlaneSurfaceKHR,
	'vkCreateDebugReportCallbackEXT':_wrap_vkCreateDebugReportCallbackEXT,
	'vkDestroyDebugReportCallbackEXT':_wrap_vkDestroyDebugReportCallbackEXT,
	'vkDebugReportMessageEXT':_wrap_vkDebugReportMessageEXT,
	'vkGetPhysicalDeviceExternalImageFormatPropertiesNV':_wrap_vkGetPhysicalDeviceExternalImageFormatPropertiesNV
    }

_device_ext_funcs = {
	'vkGetMemoryWin32HandleNV':_wrap_vkGetMemoryWin32HandleNV,
	'vkCreateSwapchainKHR':_wrap_vkCreateSwapchainKHR,
	'vkDestroySwapchainKHR':_wrap_vkDestroySwapchainKHR,
	'vkGetSwapchainImagesKHR':_wrap_vkGetSwapchainImagesKHR,
	'vkAcquireNextImageKHR':_wrap_vkAcquireNextImageKHR,
	'vkQueuePresentKHR':_wrap_vkQueuePresentKHR,
	'vkCreateSharedSwapchainsKHR':_wrap_vkCreateSharedSwapchainsKHR,
	'vkDebugMarkerSetObjectTagEXT':_wrap_vkDebugMarkerSetObjectTagEXT,
	'vkDebugMarkerSetObjectNameEXT':_wrap_vkDebugMarkerSetObjectNameEXT,
	'vkCmdDebugMarkerBeginEXT':_wrap_vkCmdDebugMarkerBeginEXT,
	'vkCmdDebugMarkerEndEXT':_wrap_vkCmdDebugMarkerEndEXT,
	'vkCmdDebugMarkerInsertEXT':_wrap_vkCmdDebugMarkerInsertEXT,
	'vkCmdDrawIndirectCountAMD':_wrap_vkCmdDrawIndirectCountAMD,
	'vkCmdDrawIndexedIndirectCountAMD':_wrap_vkCmdDrawIndexedIndirectCountAMD
    }

def vkGetInstanceProcAddr(instance, pName):
	fn = _callApi(_lib.vkGetInstanceProcAddr, instance, pName)
	if fn==ffi.NULL:
		raise ProcedureNotFoundError()
	if not pName in _instance_ext_funcs:
		raise ExtensionNotSupportedError()
	fn = ffi.cast('PFN_'+pName, fn)
	return _instance_ext_funcs[pName](fn)

def vkGetDeviceProcAddr(device, pName):
	fn = _callApi(_lib.vkGetDeviceProcAddr, device, pName)
	if fn==ffi.NULL:
		raise ProcedureNotFoundError()
	if not pName in _device_ext_funcs:
		raise ExtensionNotSupportedError()
	fn = ffi.cast('PFN_'+pName, fn)
	return _device_ext_funcs[pName](fn)